#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for eos_snmp_server
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: eos_snmp_server
short_description: Manages snmp_server resource module
description: This module configures and manages the attributes of snmp_server on Arista
  EOS platforms.
version_added: 3.2.0
author: Gomathi Selvi Srinivasan (@GomathiselviS)
notes:
- Tested against Arista EOS 4.24.6F
- This module works with connection C(network_cli) and C(httpapi).
options:
  config:
    description: SNMP server configuration.
    type: dict
    suboptions:
      chassis_id:
        description: SNMP chassis identifier.
        type: str
      communities:
        description: Community name configuration.
        type: list
        elements: dict
        suboptions:
          name:
            description: Community name
            type: str
          acl_v4:
            description: standard access_list name
            type: str
          acl_v6:
            description: IPv6 access list name.
            type: str
          ro:
            description: Only reads are permitted.
            type: bool
          rw:
            description: Read_write access
            type: bool
          view:
            description: MIB view name
            type: str
      contact:
        description: Person to contact about the syste,.
        type: str
      traps:
        description: Enable traps to all configured recipients.
        type: dict
        suboptions:
          bgp:
            description: Enable Bgp traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              arista_backward_transition:
                description: arista_backward_transition
                type: bool
              arista_established:
                description: arista_established
                type: bool
              backward_transition:
                description: backward_transition
                type: bool
              established:
                description: established.
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          bridge:
            description: Enable Bridge traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              arista_mac_age:
                description: arista_mac_age
                type: bool
              arista_mac_learn:
                description: arista_mac_learn
                type: bool
              arista_mac_move:
                description: arista_mac_move
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          capacity:
            description: Enable Capacity traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              arista_hardware_utilization_alert:
                description: arista_hardware_utilization_alert
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          entity:
            description: Enable Entity traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              arista_ent_sensor_alarm:
                description: arista_ent_sensor_alarm
                type: bool
              ent_config_change:
                description: ent_config_change
                type: bool
              ent_state_oper:
                description: ent_state_oper
                type: bool
              ent_state_oper_disabled:
                description: ent_state_oper_disabled.
                type: bool
              ent_state_oper_enabled:
                description: ent_state_oper_enabled.
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          external_alarm:
            description: Enable external alarm traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              arista_external_alarm_asserted_notif:
                description: arista_external_alarm_asserted_notif
                type: bool
              arista_external_alarm_deasserted_notif:
                description: arista_external_alarm_deasserted_notif
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          isis:
            description: Enable isis traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              adjacency_change:
                description: adjacency_change
                type: bool
              area_mismatch:
                description: area_mismatch
                type: bool
              attempt_to_exceed_max_sequence:
                description: attempt_to_exceed_max_sequence
                type: bool
              authentication_type_failure:
                description: authentication_type_failure.
                type: bool
              database_overload:
                description: database_overload
                type: bool
              own_lsp_purge:
                description: own_lsp_purge
                type: bool
              rejected_adjacency:
                description: rejected_adjacency
                type: bool
              sequence_number_skip:
                description: sequence_number_skip.
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          lldp:
            description: Enable Lldp traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              rem_tables_change:
                description: rem_tables_change
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          mpls_ldp:
            description: Enable mpls_ldp traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              mpls_ldp_session_down:
                description: mpls_ldp_session_down
                type: bool
              mpls_ldp_session_up:
                description: mpls_ldp_session_up
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          msdp:
            description: Enable msdp traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              backward_transition:
                description: backward_transition.
                type: bool
              established:
                description: established.
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          ospf:
            description: Enable Ospf traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              if_config_error:
                description: if_config_error
                type: bool
              if_auth_failure:
                description: if_auth_failure
                type: bool
              if_state_change:
                description: if_state_change
                type: bool
              nbr_state_change:
                description: nbr_state_change.
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          ospfv3:
            description: Enable Ospfv3 traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              if_config_error:
                description: if_config_error
                type: bool
              if_rx_bad_packet:
                description: if_rx_bad_packet
                type: bool
              if_state_change:
                description: if_state_change
                type: bool
              nbr_state_change:
                description: nbr_state_change.
                type: bool
              nbr_restart_helper_status_change:
                description: Enable ospfv3NbrRestartHelperStatusChange trap
                type: bool
              nssa_translator_status_change:
                description: Enable ospfv3NssaTranslatorStatusChange trap
                type: bool
              restart_status_change:
                description: restart_status_change
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          pim:
            description: Enable Pim traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              neighbor_loss:
                description: neighbor_loss
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          snmp:
            description: Enable snmp traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              authentication:
                description: authentication
                type: bool
              link_down:
                description: link_down
                type: bool
              link_up:
                description: link_up
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          snmpConfigManEvent:
            description: Enable snmpConfigManEvent traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              arista_config_man_event:
                description: arista_config_man_event
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          switchover:
            description: Enable switchover traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              arista_redundancy_switch_over_notif:
                description: arista_redundancy_switch_over_notif
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          test:
            description: Enable test traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              arista_test_notification:
                description: arista_test_notification
                type: bool
              enabled:
                description: All traps are set.
                type: bool
          vrrp:
            description: Enable vrrp traps. If set to enabled , all the traps are set.
            type: dict
            suboptions:
              trap_new_master:
                description: vrrp
                type: bool
              enabled:
                description: All traps are set.
                type: bool
      engineid:
        description: SNMPv3 engine ID configuration.
        type: dict
        suboptions:
          local:
            description:  Local SNMP agent
            type: str
          remote:
            description: Remote SNMP agent
            type: dict
            suboptions:
              host:
                description: Hostname or IP address of remote SNMP notification host
                type: str
              udp_port:
                description: The remote SNMP notification host's UDP port number.
                type: int
              id:
                description: engine ID octet string
                type: str
      extension:
        description: Configure extension script to serve an OID range
        type: dict
        suboptions:
          root_oid:
            description: Extension root oid
            type: str
          script_location:
            description: script location
            type: str
          oneshot:
            description: Use inefficient one_shot interface
            type: bool
      groups:
        description: SNMP USM group
        type: list
        elements: dict
        suboptions:
          group:
            description: SNMP group for the user
            type: str
          version:
            description: snmp security group version
            type: str
            choices: ['v1', 'v3', 'v2c']
          auth_privacy:
            description: auth and privacy config. Valid when version = v3.
            type: str
            choices: ['auth', 'noauth', 'priv']
          context:
            description: Specify a context to associate with the group
            type: str
          notify:
            description: View to restrict notifications
            type: str
          read:
            description: View to restrict read access
            type: str
          write:
            description: View to restrict write access
            type: str
      hosts:
        description: Notification destinations
        type: list
        elements: dict
        suboptions:
          host:
            description: Hostname or IP address of SNMP notification host.
            type: str
          user:
            description: Community or user name.
            type: str
          udp_port:
            description: UDP destination port for notification messages.
            type: int
          informs:
            description: Use SNMP inform messages.
            type: bool
          traps:
            description: Use SNMP trap messages
            type: bool
          version:
            description: Notification message SNMP version.
            type: str
            choices: ['1', '2c', '3 auth', '3 noauth', '3 priv']
          vrf:
            description: Specify the VRF in which the host is configured
            type: str
      acls:
        description: ipv4/ipv6 access_lists
        type: list
        elements: dict
        suboptions:
          afi:
            description: ipv4/ipv6
            type: str
            choices: ['ipv4', 'ipv6']
          acl:
            description: acl name
            type: str
          vrf:
            description: vrf name
            type: str
      local_interface:
        description: Configure the source interface for SNMP notifications.
        type: str
      location:
        description: The sysLocation string.
        type: str
      notification:
        description: Maximum number of notifications in the log
        type: int
      objects:
        description: when true Disable implementation of a group of objects
        type: dict
        suboptions:
          mac_address_tables:
            description: dot1dTpFdbTable, dot1qTpFdbTable
            type: bool
          route_tables:
            description: ipCidrRouteTable, ipCidrRouteNumber, aristaFIBStats*
            type: bool
      qos:
        description: Configure QoS parameters.
        type: int
      qosmib:
        description: Set QOS_MIB counter update interval
        type: int
      transmit:
        description: Maximum number of bytes in SNMP message (UDP/TCP payload)
        type: int
      transport:
        description: Enable snmpd transport layer protocol
        type: str
      users:
        description: SNMP user configuration.
        type: list
        elements: dict
        suboptions:
          user:
            description: SNMP user name
            type: str
          group:
            description: SNMP group for the user.
            type: str
          remote:
            description: System where an SNMPv3 user is hosted
            type: str
          udp_port:
            description: UDP port used by the remote SNMP system
            type: int
          version:
            description: snmp security version
            type: str
            choices: ['v1', 'v2c', 'v3']
          auth:
            description: User authentication settings
            type: dict
            suboptions:
              algorithm:
                description: algorithm for authentication
                type: str
              auth_passphrase:
                description: authentication passphrase hex string
                type: str
              encryption:
                description: algorithm for encryption.
                type: str
              priv_passphrase:
                description: privacy passphrase hexstring
                type: str
          localized:
            description: localized auth and privacy passphrases.
            type: dict
            suboptions:
              engineid:
                description: Engine id
                type: str
              algorithm:
                description: algorithm for authentication
                type: str
              auth_passphrase:
                description: authentication passphrase hex string
                type: str
              encryption:
                description: algorithm for encryption.
                type: str
              priv_passphrase:
                description: privacy passphrase hexstring
                type: str
      views:
        description: SNMPv2 MIB view configuration
        type: list
        elements: dict
        suboptions:
          view:
            description: SNMP view name
            type: str
          mib:
            description: SNMP MIB name
            type: str
          action:
            description: Action to be performed.
            type: str
            choices: ['excluded', 'included']
      vrfs:
        description: Specify the VRF in which the source address is used
        type: list
        elements: dict
        suboptions:
          vrf:
            description: vrf name.
            type: str
          local_interface:
            description: Configure the source interface for SNMP notifications
            type: str
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the EOS device by
      executing the command B(show running_config | section snmp_server).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  state:
    description:
    - The state the configuration should be left in.
    - The states I(replaced) and I(overridden) have identical
       behaviour for this module.
    - Please refer to examples for more details.
    type: str
    choices: [deleted, merged, overridden, replaced, gathered, rendered, parsed]
    default: merged
"""

EXAMPLES = """
# Using merged:

# Before State
# eos#show running-config | section snmp-server
# eos#

- name: merge given snmp_server configuration
  arista.eos.eos_snmp_server:
    config:
      communities:
        - name: "comm3"
          acl_v6: "list1"
          view: "view1"
        - name: "comm4"
          acl_v4: "list3"
          view: "view1"
        - name: "comm5"
          acl_v4: "list4"
          ro: true
      contact: "admin"
      engineid:
        remote:
          host: 1.1.1.1
          id: "1234567"
      groups:
        - group: "group1"
          version: "v1"
          read: "view1"
        - group: "group2"
          version: "v3"
          auth_privacy: "priv"
          notify: "view1"
          write: "view2"
      hosts:
        - host: "host02"
          user: "user01"
          udp_port: 23
          version: "2c"
        - host: "host01"
          user: "user01"
          udp_port: 23
          version: "3 priv"
      traps:
        capacity:
          arista_hardware_utilization_alert: true
        bgp:
          enabled: true
        external_alarm:
          arista_external_alarm_deasserted_notif: true
          arista_external_alarm_asserted_notif: true
      vrfs:
        - vrf: "vrf01"
          local_interface: "Ethernet1"

# After state
# eos#show running-config | section snmp-server
# snmp-server community comm3 view view1 ipv6 list1
# snmp-server community comm4 view view1 list3
# snmp-server community comm5 ro list4
# snmp-server group group1 v1 read view1
# snmp-server group group2 v3 priv write view2 notify view1
# snmp-server host host02 version 2c user01 udp-port 23
# snmp-server host host01 version 3 priv user01 udp-port 23
# snmp-server vrf vrf01 local-interface Ethernet1
# snmp-server contact admin
# snmp-server enable traps bgp
# snmp-server enable traps capacity arista-hardware-utilization-alert
# snmp-server enable traps external-alarm arista-external-alarm-asserted-notif arista-external-alarm-deasserted-notif
#
# Module Execution
#
# "after": {
#         "communities": [
#             {
#                 "acl_v6": "list1",
#                 "name": "comm3",
#                 "ro": true,
#                 "view": "view1"
#             },
#             {
#                 "acl_v4": "list3",
#                 "name": "comm4",
#                 "ro": true,
#                 "view": "view1"
#             },
#             {
#                 "acl_v4": "list4",
#                 "name": "comm5",
#                 "ro": true
#             }
#         ],
#         "contact": "admin",
#         "groups": [
#             {
#                 "group": "group1",
#                 "read": "view1",
#                 "version": "v1"
#             },
#             {
#                 "auth_privacy": "priv",
#                 "group": "group2",
#                 "notify": "view1",
#                 "version": "v3",
#                 "write": "view2"
#             }
#         ],
#         "hosts": [
#             {
#                 "host": "host01",
#                 "udp_port": 23,
#                 "user": "user01",
#                 "version": "3 priv"
#             },
#             {
#                 "host": "host02",
#                 "udp_port": 23,
#                 "user": "user01",
#                 "version": "2c"
#             }
#         ],
#         "traps": {
#             "bgp": {
#                 "enabled": true
#             },
#             "capacity": {
#                 "arista_hardware_utilization_alert": true
#             },
#             "external_alarm": {
#                 "arista_external_alarm_asserted_notif": true,
#                 "arista_external_alarm_deasserted_notif": true
#             }
#         },
#         "vrfs": [
#             {
#                 "local_interface": "Ethernet1",
#                 "vrf": "vrf01"
#             }
#         ]
#     },
#     "before": {},
#     "changed": true,
#     "commands": [
#         "snmp-server community comm3 view view1 ipv6 list1",
#         "snmp-server community comm4 view view1 list3",
#         "snmp-server community comm5 ro list4",
#         "snmp-server group group1 v1 read view1",
#         "snmp-server group group2 v3 priv write view2 notify view1",
#         "snmp-server host host02 version 2c user01 udp-port 23",
#         "snmp-server host host01 version 3 priv user01 udp-port 23",
#         "snmp-server vrf vrf01 local-interface Ethernet1",
#         "snmp-server contact admin",
#         "snmp-server engineID remote 1.1.1.1 1234567",
#         "snmp-server enable traps bgp",
#         "snmp-server enable traps capacity arista-hardware-utilization-alert",
#         "snmp-server enable traps external-alarm arista-external-alarm-asserted-notif arista-external-alarm-deasserted-notif"
#     ],
#

# Using replaced:

# Before State:
# eos#show running-config | section snmp-server
# snmp-server community comm3 view view1 ipv6 list1
# snmp-server community comm4 view view1 list3
# snmp-server community comm5 ro list4
# snmp-server group group1 v1 read view1
# snmp-server group group2 v3 priv write view2 notify view1
# snmp-server host host02 version 2c user01 udp-port 23
# snmp-server host host01 version 3 priv user01 udp-port 23
# snmp-server vrf vrf01 local-interface Ethernet1
# snmp-server contact admin
# snmp-server enable traps bgp
# snmp-server enable traps capacity arista-hardware-utilization-alert
# snmp-server enable traps external-alarm arista-external-alarm-asserted-notif arista-external-alarm-deasserted-notif

- name: Replace given snmp_server configuration
  become: true
  register: result
  arista.eos.eos_snmp_server: &replaced
    state: replaced
    config:
      communities:
        - name: "comm3"
          acl_v6: "list1"
          view: "view1"
        - name: "replacecomm"
          acl_v4: "list4"
      extension:
        root_oid: "123456"
        script_location: "flash:"
      traps:
        test:
          arista_test_notification: true
        bgp:
          enabled: true
      vrfs:
        - vrf: "vrf_replace"
          local_interface: "Ethernet1"

# After State:

# eos#show running-config | section snmp-server
# snmp-server community comm3 view view1 ipv6 list1
# snmp-server community replacecomm list4
# snmp-server vrf vrf_replace local-interface Ethernet1
# snmp-server extension 123456 flash:
# snmp-server enable traps test arista-test-notification
# snmp-server enable traps bgp

# Module Execution:
#    "after": {
#        "communities": [
#            {
#                "acl_v6": "list1",
#                "name": "comm3",
#                "ro": true,
#                "view": "view1"
#            },
#            {
#                "acl_v4": "list4",
#                "name": "replacecomm",
#                "ro": true
#            }
#        ],
#        "extension": {
#            "root_oid": "0.123456",
#            "script_location": "flash:"
#        },
#        "traps": {
#            "bgp": {
#                "enabled": true
#            },
#            "test": {
#                "arista_test_notification": true
#            }
#        },
#        "vrfs": [
#            {
#                "local_interface": "Ethernet1",
#                "vrf": "vrf_replace"
#            }
#        ]
#    },
#    "before": {
#        "communities": [
#            {
#                "acl_v6": "list1",
#                "name": "comm3",
#                "ro": true,
#                "view": "view1"
#            },
#            {
#                "acl_v4": "list3",
#                "name": "comm4",
#                "ro": true,
#                "view": "view1"
#            },
#            {
#                "acl_v4": "list4",
#                "name": "comm5",
#                "ro": true
#            }
#        ],
#        "contact": "admin",
#        "groups": [
#            {
#                "group": "group1",
#                "read": "view1",
#                "version": "v1"
#            },
#            {
#                "auth_privacy": "priv",
#                "group": "group2",
#                "notify": "view1",
#                "version": "v3",
#                "write": "view2"
#            }
#        ],
#        "hosts": [
#            {
#                "host": "host01",
#                "udp_port": 23,
#                "user": "user01",
#                "version": "3 priv"
#            },
#            {
#                "host": "host02",
#                "udp_port": 23,
#                "user": "user01",
#                "version": "2c"
#            }
#        ],
#        "traps": {
#            "bgp": {
#                "enabled": true
#            },
#            "capacity": {
#                "arista_hardware_utilization_alert": true
#            },
#            "external_alarm": {
#                "arista_external_alarm_asserted_notif": true,
#                "arista_external_alarm_deasserted_notif": true
#            }
#        },
#        "vrfs": [
#            {
#                "local_interface": "Ethernet1",
#                "vrf": "vrf01"
#            }
#        ]
#    },
#    "changed": true,
#    "commands": [
#        "snmp-server community comm3 view view1 ipv6 list1",
#        "snmp-server community replacecomm list4",
#        "no snmp-server community comm4 view view1 ro list3",
#        "no snmp-server community comm5 ro list4",
#        "no snmp-server group group1 v1 read view1",
#        "no snmp-server group group2 v3 priv write view2 notify view1",
#        "no snmp-server host host01 version 3 priv user01 udp-port 23",
#        "no snmp-server host host02 version 2c user01 udp-port 23",
#        "snmp-server vrf vrf_replace local-interface Ethernet1",
#        "no snmp-server vrf vrf01 local-interface Ethernet1",
#        "snmp-server extension 123456 flash:",
#        "default snmp-server enable traps capacity arista-hardware-utilization-alert",
#        "default snmp-server enable traps external-alarm arista-external-alarm-asserted-notif arista-external-alarm-deasserted-notif",
#        "snmp-server enable traps test arista-test-notification",
#        "no snmp-server contact admin"
#    ],

# Using overridden:
# Before State:
# eos#show running-config | section snmp-server
# snmp-server community comm3 view view1 ipv6 list1
# snmp-server community comm4 view view1 list3
# snmp-server community comm5 ro list4
# snmp-server group group1 v1 read view1
# snmp-server group group2 v3 priv write view2 notify view1
# snmp-server host host02 version 2c user01 udp-port 23
# snmp-server host host01 version 3 priv user01 udp-port 23
# snmp-server vrf vrf01 local-interface Ethernet1
# snmp-server contact admin
# snmp-server enable traps bgp
# snmp-server enable traps capacity arista-hardware-utilization-alert
# snmp-server enable traps external-alarm arista-external-alarm-asserted-notif arista-external-alarm-deasserted-notif

- name: Override given snmp_server configuration
  arista.eos.eos_snmp_server:
    state: overridden
    config:
      communities:
        - name: "comm3"
          acl_v6: "list1"
          view: "view1"
        - name: "replacecomm"
          acl_v4: "list4"
      extension:
        root_oid: "123456"
        script_location: "flash:"
      traps:
        test:
          arista_test_notification: true
        bgp:
          enabled: true
      vrfs:
        - vrf: "vrf_replace"
          local_interface: "Ethernet1"

# After State:

# eos#show running-config | section snmp-server
# snmp-server community comm3 view view1 ipv6 list1
# snmp-server community replacecomm list4
# snmp-server vrf vrf_replace local-interface Ethernet1
# snmp-server extension 123456 flash:
# snmp-server enable traps test arista-test-notification
# snmp-server enable traps bgp

# Module Execution:
#    "after": {
#        "communities": [
#            {
#                "acl_v6": "list1",
#                "name": "comm3",
#                "ro": true,
#                "view": "view1"
#            },
#            {
#                "acl_v4": "list4",
#                "name": "replacecomm",
#                "ro": true
#            }
#        ],
#        "extension": {
#            "root_oid": "0.123456",
#            "script_location": "flash:"
#        },
#        "traps": {
#            "bgp": {
#                "enabled": true
#            },
#            "test": {
#                "arista_test_notification": true
#            }
#        },
#        "vrfs": [
#            {
#                "local_interface": "Ethernet1",
#                "vrf": "vrf_replace"
#            }
#        ]
#    },
#    "before": {
#        "communities": [
#            {
#                "acl_v6": "list1",
#                "name": "comm3",
#                "ro": true,
#                "view": "view1"
#            },
#            {
#                "acl_v4": "list3",
#                "name": "comm4",
#                "ro": true,
#                "view": "view1"
#            },
#            {
#                "acl_v4": "list4",
#                "name": "comm5",
#                "ro": true
#            }
#        ],
#        "contact": "admin",
#        "groups": [
#            {
#                "group": "group1",
#                "read": "view1",
#                "version": "v1"
#            },
#            {
#                "auth_privacy": "priv",
#                "group": "group2",
#                "notify": "view1",
#                "version": "v3",
#                "write": "view2"
#            }
#        ],
#        "hosts": [
#            {
#                "host": "host01",
#                "udp_port": 23,
#                "user": "user01",
#                "version": "3 priv"
#            },
#            {
#                "host": "host02",
#                "udp_port": 23,
#                "user": "user01",
#                "version": "2c"
#            }
#        ],
#        "traps": {
#            "bgp": {
#                "enabled": true
#            },
#            "capacity": {
#                "arista_hardware_utilization_alert": true
#            },
#            "external_alarm": {
#                "arista_external_alarm_asserted_notif": true,
#                "arista_external_alarm_deasserted_notif": true
#            }
#        },
#        "vrfs": [
#            {
#                "local_interface": "Ethernet1",
#                "vrf": "vrf01"
#            }
#        ]
#    },
#    "changed": true,
#    "commands": [
#        "snmp-server community comm3 view view1 ipv6 list1",
#        "snmp-server community replacecomm list4",
#        "no snmp-server community comm4 view view1 ro list3",
#        "no snmp-server community comm5 ro list4",
#        "no snmp-server group group1 v1 read view1",
#        "no snmp-server group group2 v3 priv write view2 notify view1",
#        "no snmp-server host host01 version 3 priv user01 udp-port 23",
#        "no snmp-server host host02 version 2c user01 udp-port 23",
#        "snmp-server vrf vrf_replace local-interface Ethernet1",
#        "no snmp-server vrf vrf01 local-interface Ethernet1",
#        "snmp-server extension 123456 flash:",
#        "default snmp-server enable traps capacity arista-hardware-utilization-alert",
#        "default snmp-server enable traps external-alarm arista-external-alarm-asserted-notif arista-external-alarm-deasserted-notif",
#        "snmp-server enable traps test arista-test-notification",
#        "no snmp-server contact admin"
#    ],

# Using deleted:
# Before State:
# eos#show running-config | section snmp-server
# snmp-server community comm3 view view1 ipv6 list1
# snmp-server community comm4 view view1 list3
# snmp-server community comm5 ro list4
# snmp-server group group1 v1 read view1
# snmp-server group group2 v3 priv write view2 notify view1
# snmp-server host host02 version 2c user01 udp-port 23
# snmp-server host host01 version 3 priv user01 udp-port 23
# snmp-server vrf vrf01 local-interface Ethernet1
# snmp-server contact admin
# snmp-server enable traps bgp
# snmp-server enable traps capacity arista-hardware-utilization-alert
# snmp-server enable traps external-alarm arista-external-alarm-asserted-notif arista-external-alarm-deasserted-notif

- name: Delete given snmp_server configuration
  arista.eos.eos_snmp_server:
    state: deleted

# After State:
# eos#show running-config | section snmp-server
#

# Module Execution:
#   "after": {},
#    "before": {
#        "communities": [
#            {
#                "acl_v6": "list1",
#                "name": "comm3",
#                "ro": true,
#                "view": "view1"
#            },
#            {
#                "acl_v4": "list3",
#                "name": "comm4",
#                "ro": true,
#                "view": "view1"
#            },
#            {
#                "acl_v4": "list4",
#                "name": "comm5",
#                "ro": true
#            }
#        ],
#        "contact": "admin",
#        "groups": [
#            {
#                "group": "group1",
#                "read": "view1",
#                "version": "v1"
#            },
#            {
#                "auth_privacy": "priv",
#                "group": "group2",
#                "notify": "view1",
#                "version": "v3",
#                "write": "view2"
#            }
#        ],
#        "hosts": [
#            {
#                "host": "host01",
#                "udp_port": 23,
#                "user": "user01",
#                "version": "3 priv"
#            },
#            {
#                "host": "host02",
#                "udp_port": 23,
#                "user": "user01",
#                "version": "2c"
#            }
#        ],
#        "traps": {
#            "bgp": {
#                "enabled": true
#            },
#            "capacity": {
#                "arista_hardware_utilization_alert": true
#            },
#            "external_alarm": {
#                "arista_external_alarm_asserted_notif": true,
#                "arista_external_alarm_deasserted_notif": true
#            }
#        },
#        "vrfs": [
#            {
#                "local_interface": "Ethernet1",
#                "vrf": "vrf01"
#            }
#        ]
#    },
#    "changed": true,
#    "commands": [
#        "no snmp-server community comm3 view view1 ro ipv6 list1",
#        "no snmp-server community comm4 view view1 ro list3",
#        "no snmp-server community comm5 ro list4",
#        "no snmp-server group group1 v1 read view1",
#        "no snmp-server group group2 v3 priv write view2 notify view1",
#        "no snmp-server host host01 version 3 priv user01 udp-port 23",
#        "no snmp-server host host02 version 2c user01 udp-port 23",
#        "no snmp-server vrf vrf01 local-interface Ethernet1",
#        "no snmp-server contact admin",
#        "default snmp-server enable traps bgp",
#        "default snmp-server enable traps capacity arista-hardware-utilization-alert",
#        "default snmp-server enable traps external-alarm arista-external-alarm-asserted-notif arista-external-alarm-deasserted-notif"
#    ],
#

# Using parsed:

# _parsed.cfg
# snmp-server contact admin
# snmp-server vrf vrf01 local-interface Ethernet1
# snmp-server community comm3 view view1 ro ipv6 list1
# snmp-server community comm4 view view1 ro list3
# snmp-server community comm5 ro list4
# snmp-server group group1 v1 read view1
# snmp-server group group2 v3 priv write view2 notify view1
# snmp-server host host01 version 3 priv user01 udp-port 23
# snmp-server host host02 version 2c user01 udp-port 23
# snmp-server enable traps bgp
# snmp-server enable traps capacity arista-hardware-utilization-alert
# snmp-server enable traps external-alarm arista-external-alarm-asserted-notif
# snmp-server enable traps external-alarm arista-external-alarm-deasserted-notif

- name: Provide the running configuration for parsing (config to be parsed)
  arista.eos.eos_snmp_server:
    running_config: "{{ lookup('file', '_parsed.cfg') }}"
    state: parsed

# Module Execution:
#     "parsed": {
#         "communities": [
#             {
#                 "acl_v6": "list1",
#                 "name": "comm3",
#                 "ro": true,
#                 "view": "view1"
#             },
#             {
#                 "acl_v4": "list3",
#                 "name": "comm4",
#                 "ro": true,
#                 "view": "view1"
#             },
#             {
#                 "acl_v4": "list4",
#                 "name": "comm5",
#                 "ro": true
#             }
#         ],
#         "contact": "admin",
#         "groups": [
#             {
#                 "group": "group1",
#                 "read": "view1",
#                 "version": "v1"
#             },
#             {
#                 "auth_privacy": "priv",
#                 "group": "group2",
#                 "notify": "view1",
#                 "version": "v3",
#                 "write": "view2"
#             }
#         ],
#         "hosts": [
#             {
#                 "host": "host01",
#                 "udp_port": 23,
#                 "user": "user01",
#                 "version": "3 priv"
#             },
#             {
#                 "host": "host02",
#                 "udp_port": 23,
#                 "user": "user01",
#                 "version": "2c"
#             }
#         ],
#         "traps": {
#             "bgp": {
#                 "enabled": true
#             },
#             "capacity": {
#                 "arista_hardware_utilization_alert": true
#             },
#             "external_alarm": {
#                 "arista_external_alarm_asserted_notif": true,
#                 "arista_external_alarm_deasserted_notif": true
#             }
#         },
#         "vrfs": [
#             {
#                 "local_interface": "Ethernet1",
#                 "vrf": "vrf01"
#             }
#         ]
#   }

# Using rendered:
- name: Render given snmp_server configuration
  arista.eos.eos_snmp_server:
    state: "rendered"
    config:
      communities:
        - name: "comm3"
          acl_v6: "list1"
          view: "view1"
        - name: "comm4"
          acl_v4: "list3"
          view: "view1"
        - name: "comm5"
          acl_v4: "list4"
          ro: true
      contact: "admin"
      engineid:
        remote:
          host: 1.1.1.1
          id: "1234567"
      groups:
        - group: "group1"
          version: "v1"
          read: "view1"
        - group: "group2"
          version: "v3"
          auth_privacy: "priv"
          notify: "view1"
          write: "view2"
      hosts:
        - host: "host02"
          user: "user01"
          udp_port: 23
          version: "2c"
        - host: "host01"
          user: "user01"
          udp_port: 23
          version: "3 priv"
      traps:
        capacity:
          arista_hardware_utilization_alert: true
        bgp:
          enabled: true
        external_alarm:
          arista_external_alarm_deasserted_notif: true
          arista_external_alarm_asserted_notif: true
      vrfs:
        - vrf: "vrf01"
          local_interface: "Ethernet1"

# Module Execution:
#    "rendered": [
#        "snmp-server community comm3 view view1 ipv6 list1",
#        "snmp-server community comm4 view view1 list3",
#        "snmp-server community comm5 ro list4",
#        "snmp-server group group1 v1 read view1",
#        "snmp-server group group2 v3 priv write view2 notify view1",
#        "snmp-server host host02 version 2c user01 udp-port 23",
#        "snmp-server host host01 version 3 priv user01 udp-port 23",
#        "snmp-server vrf vrf01 local-interface Ethernet1",
#        "snmp-server contact admin",
#        "snmp-server engineID remote 1.1.1.1 1234567",
#        "snmp-server enable traps bgp",
#        "snmp-server enable traps capacity arista-hardware-utilization-alert",
#        "snmp-server enable traps external-alarm arista-external-alarm-asserted-notif arista-external-alarm-deasserted-notif"
#    ]

# using gathered:

# eos#show running-config | section snmp-server
# snmp-server community comm3 view view1 ipv6 list1
# snmp-server community comm4 view view1 list3
# snmp-server community comm5 ro list4
# snmp-server group group1 v1 read view1
# snmp-server group group2 v3 priv write view2 notify view1
# snmp-server host host02 version 2c user01 udp-port 23
# snmp-server host host01 version 3 priv user01 udp-port 23
# snmp-server vrf vrf01 local-interface Ethernet1
# snmp-server contact admin
# snmp-server enable traps bgp
# snmp-server enable traps capacity arista-hardware-utilization-alert
# snmp-server enable traps external-alarm arista-external-alarm-asserted-notif arista-external-alarm-deasserted-notif

- name: Gathered the provided configuration with the exisiting running configuration
  arista.eos.eos_snmp_server:
    config:
    state: gathered

# Module Execution:
#     "gathered": {
#         "communities": [
#             {
#                 "acl_v6": "list1",
#                 "name": "comm3",
#                 "ro": true,
#                 "view": "view1"
#             },
#             {
#                 "acl_v4": "list3",
#                 "name": "comm4",
#                 "ro": true,
#                 "view": "view1"
#             },
#             {
#                 "acl_v4": "list4",
#                 "name": "comm5",
#                 "ro": true
#             }
#         ],
#         "contact": "admin",
#         "groups": [
#             {
#                 "group": "group1",
#                 "read": "view1",
#                 "version": "v1"
#             },
#             {
#                 "auth_privacy": "priv",
#                 "group": "group2",
#                 "notify": "view1",
#                 "version": "v3",
#                 "write": "view2"
#             }
#         ],
#         "hosts": [
#             {
#                 "host": "host01",
#                 "udp_port": 23,
#                 "user": "user01",
#                 "version": "3 priv"
#             },
#             {
#                 "host": "host02",
#                 "udp_port": 23,
#                 "user": "user01",
#                 "version": "2c"
#             }
#         ],
#         "traps": {
#             "bgp": {
#                 "enabled": true
#             },
#             "capacity": {
#                 "arista_hardware_utilization_alert": true
#             },
#             "external_alarm": {
#                 "arista_external_alarm_asserted_notif": true,
#                 "arista_external_alarm_deasserted_notif": true
#             }
#         },
#         "vrfs": [
#             {
#                 "local_interface": "Ethernet1",
#                 "vrf": "vrf01"
#             }
#         ]
#     },
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - "snmp-server community comm3 view view1 ipv6 list1"
    - "snmp-server community comm4 view view1 list3"
    - "snmp-server community comm5 ro list4"
    - "snmp-server group group1 v1 read view1"
    - "snmp-server group group2 v3 priv write view2 notify view1"
    - "snmp-server host host02 version 2c user01 udp-port 23"
    - "snmp-server host host01 version 3 priv user01 udp-port 23"
    - "snmp-server vrf vrf01 local-interface Ethernet1"
    - "snmp-server contact admin"
    - "snmp-server engineID remote 1.1.1.1 1234567"
    - "snmp-server enable traps bgp"
    - "snmp-server enable traps capacity arista-hardware-utilization-alert"
    - "snmp-server enable traps external-alarm arista-external-alarm-asserted-notif arista-external-alarm-deasserted-notif"

rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - "snmp-server community comm3 view view1 ipv6 list1"
    - "snmp-server community comm4 view view1 list3"
    - "snmp-server community comm5 ro list4"
    - "snmp-server group group1 v1 read view1"
    - "snmp-server group group2 v3 priv write view2 notify view1"
    - "snmp-server host host02 version 2c user01 udp-port 23"
    - "snmp-server host host01 version 3 priv user01 udp-port 23"
    - "snmp-server vrf vrf01 local-interface Ethernet1"
    - "snmp-server contact admin"
    - "snmp-server engineID remote 1.1.1.1 1234567"
    - "snmp-server enable traps bgp"
    - "snmp-server enable traps capacity arista-hardware-utilization-alert"
    - "snmp-server enable traps external-alarm arista-external-alarm-asserted-notif arista-external-alarm-deasserted-notif"
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.arista.eos.plugins.module_utils.network.eos.argspec.snmp_server.snmp_server import (
    Snmp_serverArgs,
)
from ansible_collections.arista.eos.plugins.module_utils.network.eos.config.snmp_server.snmp_server import (
    Snmp_server,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Snmp_serverArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Snmp_server(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sda_fabric_devices_layer2_handoffs_sda_transits_v1
short_description: Resource module for Sda Fabric Devices Layer2 Handoffs Sda Transits V1
description:
- Manage operations create, update and delete of the resource Sda Fabric Devices Layer2 Handoffs Sda Transits V1.
- Adds layer 3 handoffs with sda transit in fabric devices based on user input.
- Deletes layer 3 handoffs with sda transit of a fabric device based on user input.
- Updates layer 3 handoffs with sda transit of fabric devices based on user input.
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  fabricId:
    description: FabricId query parameter. ID of the fabric this device belongs to.
    type: str
  networkDeviceId:
    description: NetworkDeviceId query parameter. Network device ID of the fabric device.
    type: str
  payload:
    description: Sda Fabric Devices Layer2 Handoffs Sda Transits's payload.
    elements: dict
    suboptions:
      affinityIdDecider:
        description: Affinity id decider value of the border node. When the affinity
          id prime value is the same on multiple devices, the affinity id decider value
          is used as a tiebreaker. Allowed range is 0-2147483647. The lower the relative
          value of affinity id decider, the higher the preference for a destination
          border node.
        type: int
      affinityIdPrime:
        description: Affinity id prime value of the border node. It supersedes the border
          priority to determine border node preference. Allowed range is 0-2147483647.
          The lower the relative value of affinity id prime, the higher the preference
          for a destination border node.
        type: int
      connectedToInternet:
        description: Set this true to allow associated site to provide internet access
          to other sites through sd-access.
        type: bool
      fabricId:
        description: ID of the fabric this device is assigned to. (updating this field
          is not allowed).
        type: str
      isMulticastOverTransitEnabled:
        description: Set this true to configure native multicast over multiple sites
          that are connected to an sd-access transit.
        type: bool
      networkDeviceId:
        description: Network device ID of the fabric device. (updating this field is
          not allowed).
        type: str
      transitNetworkId:
        description: ID of the transit network of the layer 3 handoff sda transit. (updating
          this field is not allowed).
        type: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for SDA AddFabricDevicesLayer3HandoffsWithSdaTransitV1
  description: Complete reference of the AddFabricDevicesLayer3HandoffsWithSdaTransitV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!add-fabric-devices-layer-3-handoffs-with-sda-transit
- name: Cisco DNA Center documentation for SDA DeleteFabricDeviceLayer3HandoffsWithSdaTransitV1
  description: Complete reference of the DeleteFabricDeviceLayer3HandoffsWithSdaTransitV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!delete-fabric-device-layer-3-handoffs-with-sda-transit
- name: Cisco DNA Center documentation for SDA UpdateFabricDevicesLayer3HandoffsWithSdaTransitV1
  description: Complete reference of the UpdateFabricDevicesLayer3HandoffsWithSdaTransitV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-fabric-devices-layer-3-handoffs-with-sda-transit
notes:
  - SDK Method used are
    sda.Sda.add_fabric_devices_layer3_handoffs_with_sda_transit_v1,
    sda.Sda.delete_fabric_device_layer3_handoffs_with_sda_transit_v1,
    sda.Sda.update_fabric_devices_layer3_handoffs_with_sda_transit_v1,

  - Paths used are
    post /dna/intent/api/v1/sda/fabricDevices/layer3Handoffs/sdaTransits,
    delete /dna/intent/api/v1/sda/fabricDevices/layer3Handoffs/sdaTransits,
    put /dna/intent/api/v1/sda/fabricDevices/layer3Handoffs/sdaTransits,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.sda_fabric_devices_layer2_handoffs_sda_transits_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - affinityIdDecider: 0
      affinityIdPrime: 0
      connectedToInternet: true
      fabricId: string
      isMulticastOverTransitEnabled: true
      networkDeviceId: string
      transitNetworkId: string

- name: Delete all
  cisco.dnac.sda_fabric_devices_layer2_handoffs_sda_transits_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: absent
    fabricId: string
    networkDeviceId: string

- name: Create
  cisco.dnac.sda_fabric_devices_layer2_handoffs_sda_transits_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    payload:
    - affinityIdDecider: 0
      affinityIdPrime: 0
      connectedToInternet: true
      fabricId: string
      isMulticastOverTransitEnabled: true
      networkDeviceId: string
      transitNetworkId: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: network_device_sync_v1
short_description: Resource module for Network Device Sync V1
description:
- Manage operation update of the resource Network Device Sync V1.
- >
   Synchronizes the devices. If forceSync param is false default then the sync would run in normal priority thread.
   If forceSync param is true then the sync would run in high priority thread if available, else the sync will fail.
   Result can be seen in the child task of each device.
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  forceSync:
    description: ForceSync query parameter.
    type: bool
  payload:
    description: Network Device Sync's payload.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Devices SyncDevicesV1
  description: Complete reference of the SyncDevicesV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!sync-devices
notes:
  - SDK Method used are
    devices.Devices.sync_devices_using_forcesync,

  - Paths used are
    put /dna/intent/api/v1/network-device/sync,

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.network_device_sync_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    forceSync: true
    payload:
    - string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "taskId": "string",
        "url": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
#
# Copyright (c) 2020 Sakar Mehra (@sakar97), Nikhil Patne (@nikhilpatne)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_apimanagement
version_added: "1.6.0"
short_description: Manage Azure api instances
description:
    - Create azure api instance.
    - Update the existing azure api instance.
    - Delete azure api instance.

options:
    resource_group:
        description:
          - The name of the resource group.
        required: true
        type: str
    service_name:
        description:
            - The name of the API Management service.
        required: true
        type: str
    api_id:
        description:
            - API revision identifier. It must be unique in the current API Management service instance.
        required: true
        type: str
    description:
        description:
            - Description of the API.
        type: str
    authentication_settings:
        description:
            - Collection of authentication settings included into this API.
        type: dict
        suboptions:
            o_auth2:
                description:
                    - OAuth2 Authentication settings
                type: dict
                suboptions:
                    authorization_server_id:
                        description:
                            - OAuth authorization server identifier.
                        type: str
                    scope:
                        description:
                            - operations scope.
                        type: str
            openid:
                description:
                    - OpenID Connect Authentication Settings
                type: dict
                suboptions:
                    openid_provider_id:
                        description:
                            - OAuth authorization server identifier.
                        type: str
                    bearer_token_sending_methods:
                        description:
                            - How to send token to the server.
                        type: list
                        elements: str
                        choices:
                            - authorizationHeader
                            - query
    subscription_key_parameter_names:
        description:
            - Protocols over which API is made available.
        type: dict
        suboptions:
            header:
                description:
                    - Subscription key header name.
                type: str
            query:
                description:
                    - Subscription key query string parameter name.
                type: str
    type:
        description:
            - Type of API
        type: str
        choices:
            - http
            - soap
    api_revision:
        description:
            - Describes the Revision of the Api.
            - If no value is provided, default revision 1 is created
        type: str
    api_version:
        description:
            - Indicates the Version identifier of the API if the API is versioned
        type: str
    is_current:
        description:
            - Indicates if API revision is current api revision.
        type: bool
    api_revision_description:
        description:
            - Description of the Api Revision.
        type: str
    api_version_description:
        description:
            - Description of the Api Version.
        type: str
    api_version_set_id:
        description:
            - A resource identifier for the related ApiVersionSet.
        type: str
    subscription_required:
        description:
            - Specifies whether an API or Product subscription is required for accessing the API.
        type: bool
    source_api_id:
        description:
            - API identifier of the source API.
        type: str
    display_name:
        description:
            - API Name to be displayed. It must be 1 to 300 characters long.
        type: str
    service_url:
        description:
            - Absolute URL of the backend service implementing this API
            - Cannot be more than 2000 characters long.
        type: str
    path:
        description:
            - Relative URL uniquely identifying this API.
        type: str
    protocols:
        description:
            - Describes on which protocols the operations in this API can be invoked.
        type: list
        elements: str
        choices:
            - http
            - https
    api_version_set:
        description:
            - Version set details
        type: dict
        suboptions:
            id:
                description:
                    - Identifier for existing API Version Set
                    - Omit this value to create a new Version Set.
                type: str
            name:
                description:
                    - The display Name of the API Version Set.
                type: str
            description:
                description:
                    - Description of API Version Set.
                type: str
            versioning_scheme:
                description:
                    - An value that determines where the API Version identifer will be located in a HTTP request.
                type: str
                choices:
                    - Segment
                    - Query
                    - Header
            version_query_name:
                description:
                    - Name of query parameter that indicates the API Version if versioningScheme is set to `query`.
                type: str
            version_header_name:
                description:
                    - Name of HTTP header parameter that indicates the API Version if versioningScheme is set to `header`.
                type: str
    value:
        description:
            - Content value when Importing an API.
        type: str
    format:
        description:
            - Format of the Content in which the API is getting imported.
        type: str
        choices:
            - wadl-xml
            - wadl-link-json
            - swagger-json
            - swagger-link-json
            - wsdl
            - wsdl-link
            - openapi
            - openapi+json
            - openapi-link
    wsdl_selector:
        description:
            - Criteria to limit import of WSDL to a subset of the document.
        type: dict
        suboptions:
            wsdl_service_name:
                description:
                    - Name of service to import from WSDL.
                type: str
            wsdl_endpoint_name:
                description:
                    - Name of endpoint(port) to import from WSDL.
                type: str
    api_type:
        description:
            - Type of Api to create.
            - C(http) creates a SOAP to REST API.
            - C(soap) creates a SOAP pass-through API.
        type: str
        choices:
            - soap
            - http
    state:
        description:
            - State of the Api.
            - Use C(present) to create or update an Api and C(absent) to delete it.
        type: str
        default: present
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Sakar Mehra (@sakar97)
    - Nikhil Patne (@nikhilpatne)

'''

EXAMPLES = '''
- name: Create a new API instance
  azure_rm_apimanagement:
    resource_group: 'myResourceGroup'
    service_name: myService
    api_id: testApi
    description: testDescription
    display_name: TestAPI
    service_url: 'http://testapi.example.net/api'
    path: myapiPath
    protocols:
      - https
- name: Update an existing API instance.
  azure_rm_apimanagement:
    resource_group: myResourceGroup
    service_name: myService
    api_id: testApi
    display_name: newTestAPI
    service_url: 'http://testapi.example.net/api'
    path: myapiPath
    protocols:
      - https
- name: ApiManagementDeleteApi
  azure_rm_apimanagement:
    resource_group: myResourceGroup
    service_name: myService
    api_id: testApi
    state: absent
'''

RETURN = \
    '''
id:
    description:
        - Resource ID.
    returned: always
    type: str
    sample: null
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_rest import GenericRestClient
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
import json


class Actions:
    NoAction, Create, Update, Delete = range(4)


class AzureApiManagement(AzureRMModuleBaseExt):
    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            service_name=dict(
                type='str',
                required=True
            ),
            api_id=dict(
                type='str',
                required=True
            ),
            description=dict(
                type='str',
            ),
            authentication_settings=dict(
                type='dict',
                options=dict(
                    o_auth2=dict(
                        type='dict',
                        options=dict(
                            authorization_server_id=dict(
                                type='str',
                            ),
                            scope=dict(
                                type='str',
                            )
                        )
                    ),
                    openid=dict(
                        type='dict',
                        options=dict(
                            openid_provider_id=dict(
                                type='str',
                            ),
                            bearer_token_sending_methods=dict(
                                type='list',
                                elements='str',
                                choices=['authorizationHeader', 'query']
                            )
                        )
                    )
                )
            ),
            subscription_key_parameter_names=dict(
                type='dict',
                no_log=True,
                options=dict(
                    header=dict(
                        type='str',
                        required=False
                    ),
                    query=dict(
                        type='str',
                        required=False
                    )
                )
            ),
            type=dict(
                type='str',
                choices=['http', 'soap']
            ),
            api_revision=dict(
                type='str',
            ),
            api_version=dict(
                type='str',
            ),
            is_current=dict(
                type='bool',
            ),
            api_revision_description=dict(
                type='str',
            ),
            api_version_description=dict(
                type='str',
            ),
            api_version_set_id=dict(
                type='str',
            ),
            subscription_required=dict(
                type='bool',
            ),
            source_api_id=dict(
                type='str',
            ),
            display_name=dict(
                type='str',
            ),
            service_url=dict(
                type='str',
            ),
            path=dict(
                type='str',
            ),
            protocols=dict(
                type='list',
                elements='str',
                choices=['http',
                         'https']
            ),
            api_version_set=dict(
                type='dict',
                options=dict(
                    id=dict(
                        type='str'
                    ),
                    name=dict(
                        type='str'
                    ),
                    description=dict(
                        type='str'
                    ),
                    versioning_scheme=dict(
                        type='str',
                        choices=['Segment',
                                 'Query',
                                 'Header']
                    ),
                    version_query_name=dict(
                        type='str',
                    ),
                    version_header_name=dict(
                        type='str',
                    )
                )
            ),
            value=dict(
                type='str',
            ),
            format=dict(
                type='str',
                choices=['wadl-xml',
                         'wadl-link-json',
                         'swagger-json',
                         'swagger-link-json',
                         'wsdl',
                         'wsdl-link',
                         'openapi',
                         'openapi+json',
                         'openapi-link']
            ),
            wsdl_selector=dict(
                type='dict',
                options=dict(
                    wsdl_service_name=dict(
                        type='str',
                    ),
                    wsdl_endpoint_name=dict(
                        type='str',
                    )
                )
            ),
            api_type=dict(
                type='str',
                choices=['http', 'soap']
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.service_name = None
        self.api_id = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.url = None
        self.status_code = [200, 201, 202]
        self.to_do = Actions.NoAction

        self.body = {}
        self.body['properties'] = {}
        self.query_parameters = {}
        self.query_parameters['api-version'] = '2022-08-01'
        self.header_parameters = {}
        self.header_parameters['Content-Type'] = 'application/json; charset=utf-8'

        super(AzureApiManagement, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                 supports_check_mode=True,
                                                 supports_tags=True)

    def get_url(self):
        return '/subscriptions' + '/' + self.subscription_id \
               + '/resourceGroups' + '/' + self.resource_group \
               + '/providers' + '/Microsoft.ApiManagement' + '/service' \
               + '/' + self.service_name + '/apis' + '/' + self.api_id

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                if key == 'description':
                    self.body['properties']['description'] = kwargs[key]
                elif key == 'authentication_settings':
                    self.body['properties']['authenticationSettings'] = {}
                    if kwargs[key].get('o_auth2') is not None:
                        self.body['properties']['authenticationSettings']['oAuth2'] = {}
                        for item in kwargs[key]['o_auth2'].keys():
                            if item == 'authorization_server_id':
                                authorization_id = kwargs[key]['o_auth2']['authorization_server_id']
                                self.body['properties']['authenticationSettings']['oAuth2']['authorizationServerId'] = authorization_id
                            elif item == 'scope':
                                self.body['properties']['authenticationSettings']['oAuth2']['scope'] = kwargs[key]['o_auth2']['scope']
                    elif kwargs[key].get('openid') is not None:
                        self.body['properties']['authenticationSettings']['openid'] = {}
                        for item in kwargs[key]['openid'].keys():
                            if item == 'openid_provider_id' and kwargs[key]['openid'].get('openid_provider_id') is not None:
                                openid_pro = kwargs[key]['openid'].get('openid_provider_id')
                                self.body['properties']['authenticationSettings']['openid']['openidProviderId'] = openid_pro
                            elif item == 'bearer_token_sending_methods' and kwargs[key]['openid'].get('bearer_token_sending_methods') is not None:
                                bearer_token = kwargs[key]['openid']['bearer_token_sending_methods']
                                self.body['properties']['authenticationSettings']['openid']['bearerTokenSendingMethods'] = bearer_token
                elif key == 'subscription_key_parameter_names':
                    self.body['properties']['subscriptionKeyParameterNames'] = kwargs[key]
                elif key == 'type':
                    self.body['properties']['type'] = kwargs[key]
                elif key == 'api_revision':
                    self.body['properties']['apiRevision'] = kwargs[key]
                elif key == 'api_version':
                    self.body['properties']['apiVersion'] = kwargs[key]
                elif key == 'is_current':
                    self.body['properties']['isCurrent'] = kwargs[key]
                elif key == 'api_revision_description':
                    self.body['properties']['apiRevisionDescription'] = kwargs[key]
                elif key == 'api_version_description':
                    self.body['properties']['apiVersionDescription'] = kwargs[key]
                elif key == 'api_version_set_id':
                    self.body['properties']['apiVersionSetId'] = kwargs[key]
                elif key == 'subscription_required':
                    self.body['properties']['subscriptionRequired'] = kwargs[key]
                elif key == 'source_api_id':
                    self.body['properties']['sourceApiId'] = kwargs[key]
                elif key == 'display_name':
                    self.body['properties']['displayName'] = kwargs[key]
                elif key == 'service_url':
                    self.body['properties']['serviceUrl'] = kwargs[key]
                elif key == 'protocols':
                    self.body['properties']['protocols'] = kwargs[key]
                elif key == 'api_version_set':
                    self.body['properties']['apiVersionSet'] = {}
                    for item in kwargs[key].keys():
                        if item == 'versioning_scheme':
                            self.body['properties']['apiVersionSet'] = kwargs[key].get('versioning_scheme')
                        elif item == 'version_query_name':
                            self.body['properties']['versionQueryName'] = kwargs[key].get('version_query_name')
                        elif item == 'version_header_name':
                            self.body['properties']['versionHeaderName'] = kwargs[key].get('version_header_name')
                        else:
                            self.body['properties'][item] = kwargs[key].get(item)
                elif key == 'wsdl_selector':
                    self.body['properties']['wsdlSelector'] = {}
                    for item in kwargs[key].keys():
                        if item == 'wsdl_service_name':
                            self.body['properties']['wsdlSelector']['wsdlServiceName'] = kwargs[key].get(item)
                        if item == 'wsdl_endpoint_name':
                            self.body['properties']['wsdlSelector']['wsdlEndpointName'] = kwargs[key].get(item)
                elif key == 'api_type':
                    self.body['properties']['apiType'] = kwargs[key]
                else:
                    self.body['properties'][key] = kwargs[key]

        self.url = self.get_url()
        old_response = None
        response = None

        self.mgmt_client = self.get_mgmt_svc_client(GenericRestClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager)

        old_response = self.get_resource()

        if not old_response:
            self.log("Api instance does not exist in the given service.")
            if self.state == 'present':
                self.to_do = Actions.Create
            else:
                self.log("Old instance didn't exist")
        else:
            self.log("Api instance already exists in the given service.")
            if self.state == 'absent':
                self.to_do = Actions.Delete
            else:
                if self.body['properties'].get('description') is not None and \
                   self.body['properties']['description'] != old_response['properties']['description']:
                    self.to_do = Actions.Update
                elif self.body['properties'].get('authenticationSettings') is not None:
                    if old_response['properties'].get('authenticationSettings') is None:
                        self.to_do = Actions.Update
                    elif (self.body['properties']['authenticationSettings'].get('oAuth2') is not None and
                          self.body['properties']['authenticationSettings']['oAuth2'] != old_response['properties']['authenticationSettings'].get('oAuth2')):
                        self.to_do = Actions.Update
                    elif (self.body['properties']['authenticationSettings'].get('openid') is not None and
                          self.body['properties']['authenticationSettings']['openid'] != old_response['properties']['authenticationSettings'].get('openid')):
                        self.to_do = Actions.Update
                elif self.body['properties'].get('subscriptionKeyParameterNames') is not None:
                    tt = old_response['properties']
                    if old_response['properties'].get('subscriptionKeyParameterNames') is None:
                        self.to_do = Actions.Update
                    elif (not all(self.body['properties']['subscriptionKeyParameterNames'].get(item) == tt['subscriptionKeyParameterNames'].get(item)
                          for item in self.body['properties']['subscriptionKeyParameterNames'].keys())):
                        self.to_do = Actions.Update
                elif (self.body['properties'].get('apiRevision') is not None and
                      self.body['properties']['apiRevision'] != old_response['properties'].get('apiRevision')):
                    self.to_do = Actions.Update
                elif (self.body['properties'].get('apiVersion') is not None and
                      self.body['properties']['apiVersion'] != old_response['properties'].get('apiVersion')):
                    self.to_do == Actions.Update
                elif (self.body['properties'].get('isCurrent') is not None and
                      self.body['properties']['isCurrent'] != old_response['properties'].get('isCurrent')):
                    self.to_do = Actions.Update
                elif (self.body['properties'].get('apiRevisionDescription') is not None and self.body['properties']['apiRevisionDescription'] !=
                      old_response['properties'].get('apiRevisionDescription')):
                    self.to_do = Actions.Update
                elif (self.body['properties'].get('apiVersionDescription') is not None and
                      self.body['properties']['apiVersionDescription'] != old_response['properties'].get('apiVersionDescription')):
                    self.to_do = Actions.Update
                elif (self.body['properties'].get('apiVersionSetId') is not None and
                      self.body['properties']['apiVersionSetId'] != old_response['properties'].get('apiVersionSetId')):
                    self.to_do = Actions.Update
                elif (self.body['properties'].get('subscriptionRequired') is not None and
                      self.body['properties']['subscriptionRequired'] != old_response['properties'].get('subscriptionRequired')):
                    self.to_do = Actions.Update
                elif (self.body['properties'].get('sourceApiId') is not None and
                      self.body['properties']['sourceApiId'] != old_response['properties'].get('sourceApiId')):
                    self.to_do = Actions.Update
                elif (self.body['properties'].get('displayName') is not None and
                      self.body['properties']['displayName'] != old_response['properties'].get('displayName')):
                    self.to_do = Actions.Update
                elif (self.body['properties'].get('serviceUrl') is not None and
                      self.body['properties']['serviceUrl'] != old_response['properties'].get('serviceUrl')):
                    self.to_do = Actions.Update
                elif self.body['properties'].get('path') is not None and self.body['properties']['path'] != old_response['properties'].get('path'):
                    self.to_do = Actions.Update
                elif (self.body['properties'].get('protocols') is not None and
                      self.body['properties']['protocols'] != old_response['properties'].get('protocols')):
                    self.to_do = Actions.Update
                elif self.body['properties'].get('type') is not None and self.body['properties']['type'] != old_response['properties'].get('type'):
                    self.to_do = Actions.Update
                elif self.body['properties'].get('apiType') is not None and self.body['properties']['apiType'] != old_response['properties'].get('apiType'):
                    self.to_do = Actions.Update
                elif self.body['properties'].get('value') is not None and self.body['properties']['value'] != old_response['properties'].get('value'):
                    self.to_do = Actions.Update
                elif self.body['properties'].get('format') is not None and self.body['properties']['format'] != old_response['properties'].get('format'):
                    self.to_do = Actions.Update
                elif (self.body['properties'].get('wsdlSelector') is not None and
                      not all(self.body['properties']['wsdlSelector'][item] == old_response['properties']['wsdlSelector'].get(item)
                      for item in self.body['properties']['wsdlSelector'].keys())):
                    self.to_do = Actions.Update
                elif (self.body['properties'].get('apiVersionSet') is not None and
                      not all(self.body['properties']['apiVersionSet'][item] == old_response['properties']['apiVersionSet'].get(item)
                      for item in self.body['properties']['apiVersionSet'].keys())):
                    self.to_do = Actions.Update

        if (self.to_do == Actions.Create) or (self.to_do == Actions.Update):
            self.log('Create and Update the Api instance.')

            if self.check_mode:
                self.results['changed'] = True
                return self.results

            response = self.create_and_update_resource()
            self.results['changed'] = True

        elif self.to_do == Actions.Delete:
            self.log('Api instance deleted.')

            if self.check_mode:
                self.results['changed'] = True
                return self.results

            self.delete_resource()
            self.results['changed'] = True
        else:
            self.log('No change in Api instance.')
            self.results['changed'] = False
            response = old_response

        if response:
            self.results["id"] = response["id"]

        return self.results

    # This function will create and update resource on the api management service.
    def create_and_update_resource(self):

        try:
            response = self.mgmt_client.query(
                self.url,
                'PUT',
                self.query_parameters,
                self.header_parameters,
                self.body,
                self.status_code,
                600,
                30,
            )
        except Exception as exc:
            self.log('Error while creating/updating the Api instance.')
            self.fail('Error creating the Api instance: {0}'.format(str(exc)))
        if hasattr(response, 'body'):
            response = json.loads(response.body())
        elif hasattr(response, 'context'):
            response = response.context['deserialized_data']
        else:
            self.fail("Create or Updating fail, no match message return, return info as {0}".format(response))

        return response

    def delete_resource(self):
        isDeleted = False
        try:
            response = self.mgmt_client.query(
                self.url,
                'DELETE',
                self.query_parameters,
                self.header_parameters,
                None,
                self.status_code,
                600,
                30,
            )
            isDeleted = True
        except Exception as e:
            self.log('Error attempting to delete the Api instance.')
            self.fail('Error deleting the Api instance: {0}'.format(str(e)))

        return isDeleted

    def get_resource(self):
        isFound = False
        try:
            response = self.mgmt_client.query(
                self.url,
                'GET',
                self.query_parameters,
                self.header_parameters,
                None,
                self.status_code,
                600,
                30,
            )
            isFound = True
            response = json.loads(response.body())
            self.log("Response : {0}".format(response))
        except Exception as e:
            self.log('Could not find the Api instance from the given parameters.')
        if isFound is True:
            return response
        return False


def main():
    AzureApiManagement()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: license_setting_v1_info
short_description: Information module for License Setting V1
description:
- Get all License Setting V1.
- >
   Retrieves license setting - Default smart account id and virtual account id for auto registration of devices for
   smart license flow. If default smart account is not configured, 'defaultSmartAccountId' is 'null'. Similarly, if
   auto registration of devices for smart license flow is not enabled, 'autoRegistrationVirtualAccountId' is 'null'.
   For smart proxy connection mode, 'autoRegistrationVirtualAccountId' is always 'null'.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module_info
author: Rafael Campos (@racampos)
options:
  headers:
    description: Additional headers.
    type: dict
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Licenses RetrieveLicenseSettingV1
  description: Complete reference of the RetrieveLicenseSettingV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!retrieve-license-setting
notes:
  - SDK Method used are
    licenses.Licenses.retrieve_license_setting_v1,

  - Paths used are
    get /dna/intent/api/v1/licenseSetting,

"""

EXAMPLES = r"""
- name: Get all License Setting V1
  cisco.dnac.license_setting_v1_info:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    headers: "{{my_headers | from_json}}"
  register: result

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "response": {
        "defaultSmartAccountId": "string",
        "autoRegistrationVirtualAccountId": "string"
      },
      "version": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: integrate_ise
short_description: Resource module for Integrate Ise
description:
- This module represents an alias of the module integrate_ise_v1
version_added: '6.14.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  id:
    description: Id path parameter. Cisco ISE Server Identifier. Use 'Get Authentication
      and Policy Servers' intent API to find the identifier.
    type: str
  isCertAcceptedByUser:
    description: Value true for accept, false for deny. Remove this field and send empty
      request payload ( {} ) to retry the failed integration.
    type: bool
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for System Settings AcceptCiscoISEServerCertificateForCiscoISEServerIntegrationV1
  description: Complete reference of the AcceptCiscoISEServerCertificateForCiscoISEServerIntegrationV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!accept-cisco-ise-server-certificate-for-cisco-ise-server-integration
notes:
  - SDK Method used are
    system_settings.SystemSettings.accept_cisco_ise_server_certificate_for_cisco_ise_server_integration_v1,

  - Paths used are
    put /dna/intent/api/v1/integrate-ise/{id},
  - It should be noted that this module is an alias of integrate_ise_v1

"""

EXAMPLES = r"""
- name: Update by id
  cisco.dnac.integrate_ise:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    id: string
    isCertAcceptedByUser: true

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "object": "string"
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: telemetry_settings_apply_v1
short_description: Resource module for Telemetry Settings Apply V1
description:
- Manage operation create of the resource Telemetry Settings Apply V1.
- >
   Update a devices telemetry settings to conform to the telemetry settings for its site. One Task is created to
   track the update, for more granular status tracking, split your devices into multiple requests.
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  deviceIds:
    description: The list of device Ids to perform the provisioning against.
    elements: str
    type: list
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Network Settings UpdateADevicesTelemetrySettingsToConformToTheTelemetrySettingsForItsSiteV1
  description: Complete reference of the UpdateADevicesTelemetrySettingsToConformToTheTelemetrySettingsForItsSiteV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-a-devices-telemetry-settings-to-conform-to-the-telemetry-settings-for-its-site
notes:
  - SDK Method used are
    network_settings.NetworkSettings.update_a_devices_telemetry_settings_to_conform_to_the_telemetry_settings_for_its_site_v1,

  - Paths used are
    post /dna/intent/api/v1/telemetrySettings/apply,

"""

EXAMPLES = r"""
- name: Create
  cisco.dnac.telemetry_settings_apply_v1:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    deviceIds:
    - string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: network_devices_device_controllability_settings
short_description: Resource module for Network Devices Device Controllability Settings
description:
- This module represents an alias of the module network_devices_device_controllability_settings_v1
version_added: '6.15.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  autocorrectTelemetryConfig:
    description: If it is true, autocorrect telemetry config is enabled. If it is false,
      autocorrect telemetry config is disabled. The autocorrect telemetry config feature
      is supported only when device controllability is enabled.
    type: bool
  deviceControllability:
    description: If it is true, device controllability is enabled. If it is false, device
      controllability is disabled.
    type: bool
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Site Design UpdateDeviceControllabilitySettingsV1
  description: Complete reference of the UpdateDeviceControllabilitySettingsV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!update-device-controllability-settings
notes:
  - SDK Method used are
    site_design.SiteDesign.update_device_controllability_settings_v1,

  - Paths used are
    put /dna/intent/api/v1/networkDevices/deviceControllability/settings,
  - It should be noted that this module is an alias of network_devices_device_controllability_settings_v1

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.network_devices_device_controllability_settings:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    state: present
    autocorrectTelemetryConfig: true
    deviceControllability: true

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "url": "string",
        "taskId": "string"
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)


DOCUMENTATION = r"""
---
module: sensor_test_template_duplicate
short_description: Resource module for Sensor Test Template Duplicate
description:
- This module represents an alias of the module sensor_test_template_duplicate_v1
version_added: '3.1.0'
extends_documentation_fragment:
  - cisco.dnac.module
author: Rafael Campos (@racampos)
options:
  newTemplateName:
    description: Destination test template name.
    type: str
  templateName:
    description: Source test template name.
    type: str
requirements:
- dnacentersdk >= 2.4.9
- python >= 3.5
seealso:
- name: Cisco DNA Center documentation for Sensors DuplicateSensorTestTemplateV1
  description: Complete reference of the DuplicateSensorTestTemplateV1 API.
  link: https://developer.cisco.com/docs/dna-center/#!duplicate-sensor-test-template
notes:
  - SDK Method used are
    sensors.Sensors.duplicate_sensor_test_template_v1,

  - Paths used are
    put /dna/intent/api/v1/sensorTestTemplate,
  - It should be noted that this module is an alias of sensor_test_template_duplicate_v1

"""

EXAMPLES = r"""
- name: Update all
  cisco.dnac.sensor_test_template_duplicate:
    dnac_host: "{{dnac_host}}"
    dnac_username: "{{dnac_username}}"
    dnac_password: "{{dnac_password}}"
    dnac_verify: "{{dnac_verify}}"
    dnac_port: "{{dnac_port}}"
    dnac_version: "{{dnac_version}}"
    dnac_debug: "{{dnac_debug}}"
    newTemplateName: string
    templateName: string

"""
RETURN = r"""
dnac_response:
  description: A dictionary or list with the response returned by the Cisco DNAC Python SDK
  returned: always
  type: dict
  sample: >
    {
      "version": "string",
      "response": {
        "name": "string",
        "_id": "string",
        "version": 0,
        "modelVersion": 0,
        "startTime": 0,
        "lastModifiedTime": 0,
        "numAssociatedSensor": 0,
        "location": "string",
        "siteHierarchy": "string",
        "status": "string",
        "connection": "string",
        "actionInProgress": "string",
        "frequency": {
          "value": 0,
          "unit": "string"
        },
        "rssiThreshold": 0,
        "numNeighborAPThreshold": 0,
        "scheduleInDays": 0,
        "wlans": [
          "string"
        ],
        "ssids": [
          {
            "bands": "string",
            "ssid": "string",
            "profileName": "string",
            "numAps": 0,
            "numSensors": 0,
            "layer3webAuthsecurity": "string",
            "layer3webAuthuserName": "string",
            "layer3webAuthpassword": "string",
            "layer3webAuthEmailAddress": "string",
            "thirdParty": {
              "selected": true
            },
            "id": 0,
            "wlanId": 0,
            "wlc": "string",
            "validFrom": 0,
            "validTo": 0,
            "status": "string",
            "proxyServer": "string",
            "proxyPort": "string",
            "proxyUserName": "string",
            "proxyPassword": "string",
            "authType": "string",
            "psk": "string",
            "username": "string",
            "password": "string",
            "passwordType": "string",
            "eapMethod": "string",
            "scep": true,
            "authProtocol": "string",
            "certfilename": "string",
            "certxferprotocol": "string",
            "certstatus": "string",
            "certpassphrase": "string",
            "certdownloadurl": "string",
            "extWebAuthVirtualIp": "string",
            "extWebAuth": true,
            "whiteList": true,
            "extWebAuthPortal": "string",
            "extWebAuthAccessUrl": "string",
            "extWebAuthHtmlTag": [
              {
                "label": "string",
                "tag": "string",
                "value": "string"
              }
            ],
            "qosPolicy": "string",
            "tests": [
              {
                "name": "string",
                "config": [
                  {
                    "domains": [
                      "string"
                    ],
                    "server": "string",
                    "userName": "string",
                    "password": "string",
                    "url": "string",
                    "port": 0,
                    "protocol": "string",
                    "servers": [
                      "string"
                    ],
                    "direction": "string",
                    "startPort": 0,
                    "endPort": 0,
                    "udpBandwidth": 0,
                    "probeType": "string",
                    "numPackets": 0,
                    "pathToDownload": "string",
                    "transferType": "string",
                    "sharedSecret": "string",
                    "ndtServer": "string",
                    "ndtServerPort": "string",
                    "ndtServerPath": "string",
                    "uplinkTest": true,
                    "downlinkTest": true,
                    "proxyServer": "string",
                    "proxyPort": "string",
                    "proxyUserName": "string",
                    "proxyPassword": "string",
                    "userNamePrompt": "string",
                    "passwordPrompt": "string",
                    "exitCommand": "string",
                    "finalPrompt": "string"
                  }
                ]
              }
            ]
          }
        ],
        "profiles": [
          {
            "authType": "string",
            "psk": "string",
            "username": "string",
            "password": "string",
            "passwordType": "string",
            "eapMethod": "string",
            "scep": true,
            "authProtocol": "string",
            "certfilename": "string",
            "certxferprotocol": "string",
            "certstatus": "string",
            "certpassphrase": "string",
            "certdownloadurl": "string",
            "extWebAuthVirtualIp": "string",
            "extWebAuth": true,
            "whiteList": true,
            "extWebAuthPortal": "string",
            "extWebAuthAccessUrl": "string",
            "extWebAuthHtmlTag": [
              {
                "label": "string",
                "tag": "string",
                "value": "string"
              }
            ],
            "qosPolicy": "string",
            "tests": [
              {
                "name": "string",
                "config": [
                  {
                    "domains": [
                      "string"
                    ],
                    "server": "string",
                    "userName": "string",
                    "password": "string",
                    "url": "string",
                    "port": 0,
                    "protocol": "string",
                    "servers": [
                      "string"
                    ],
                    "direction": "string",
                    "startPort": 0,
                    "endPort": 0,
                    "udpBandwidth": 0,
                    "probeType": "string",
                    "numPackets": 0,
                    "pathToDownload": "string",
                    "transferType": "string",
                    "sharedSecret": "string",
                    "ndtServer": "string",
                    "ndtServerPort": "string",
                    "ndtServerPath": "string",
                    "uplinkTest": true,
                    "downlinkTest": true,
                    "proxyServer": "string",
                    "proxyPort": "string",
                    "proxyUserName": "string",
                    "proxyPassword": "string",
                    "userNamePrompt": "string",
                    "passwordPrompt": "string",
                    "exitCommand": "string",
                    "finalPrompt": "string"
                  }
                ]
              }
            ],
            "profileName": "string",
            "deviceType": "string",
            "vlan": "string",
            "locationVlanList": [
              {
                "locationId": "string",
                "vlans": [
                  "string"
                ]
              }
            ]
          }
        ],
        "testScheduleMode": "string",
        "showWlcUpgradeBanner": true,
        "radioAsSensorRemoved": true,
        "encryptionMode": "string",
        "runNow": "string",
        "locationInfoList": [
          {
            "locationId": "string",
            "locationType": "string",
            "allSensors": true,
            "siteHierarchy": "string",
            "macAddressList": [
              "string"
            ],
            "managementVlan": "string",
            "customManagementVlan": true
          }
        ],
        "sensors": [
          {
            "name": "string",
            "macAddress": "string",
            "switchMac": "string",
            "switchUuid": "string",
            "switchSerialNumber": "string",
            "markedForUninstall": true,
            "ipAddress": "string",
            "hostName": "string",
            "wiredApplicationStatus": "string",
            "wiredApplicationMessage": "string",
            "assigned": true,
            "status": "string",
            "xorSensor": true,
            "targetAPs": [
              "string"
            ],
            "runNow": "string",
            "locationId": "string",
            "allSensorAddition": true,
            "configUpdated": "string",
            "sensorType": "string",
            "testMacAddresses": {},
            "id": "string",
            "servicePolicy": "string",
            "iPerfInfo": {}
          }
        ],
        "apCoverage": [
          {
            "bands": "string",
            "numberOfApsToTest": 0,
            "rssiThreshold": 0
          }
        ]
      }
    }
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sg_mapping_group_deploy_status_info
short_description: Information module for SG Mapping Group Deploy Status
description:
- Get all SG Mapping Group Deploy Status.
- This API allows the client to get the IP to SGT mapping group deployment.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.ise.module_info
author: Rafael Campos (@racampos)
options: {}
requirements:
- ciscoisesdk >= 2.2.3
- python >= 3.5
seealso:
- name: Cisco ISE documentation for IPToSGTMappingGroup
  description: Complete reference of the IPToSGTMappingGroup API.
  link: https://developer.cisco.com/docs/identity-services-engine/v1/#!sgmappinggroup
notes:
  - SDK Method used are
    ip_to_sgt_mapping_group.IpToSgtMappingGroup.get_deploy_status_ip_to_sgt_mapping_group,

  - Paths used are
    put /ers/config/sgmappinggroup/deploy/status,

"""

EXAMPLES = r"""
- name: Get all SG Mapping Group Deploy Status
  cisco.ise.sg_mapping_group_deploy_status_info:
    ise_hostname: "{{ise_hostname}}"
    ise_username: "{{ise_username}}"
    ise_password: "{{ise_password}}"
    ise_verify: "{{ise_verify}}"
  register: result

"""

RETURN = r"""
ise_response:
  description: A dictionary or list with the response returned by the Cisco ISE Python SDK
  returned: always
  type: dict
  sample: >
    {
      "resultValue": [
        {
          "value": "string",
          "name": "string"
        }
      ]
    }
"""



# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
filter plugin file for ipaddr filters: ipv6
"""
from __future__ import absolute_import, division, print_function

from functools import partial

from ansible.errors import AnsibleFilterError

from ansible_collections.ansible.utils.plugins.module_utils.common.argspec_validate import (
    AnsibleArgSpecValidator,
)
from ansible_collections.ansible.utils.plugins.plugin_utils.base.ipaddr_utils import (
    _need_netaddr,
    ipaddr,
)


__metaclass__ = type


try:
    from jinja2.filters import pass_environment
except ImportError:
    from jinja2.filters import environmentfilter as pass_environment

try:
    import netaddr

    HAS_NETADDR = True
except ImportError:
    # in this case, we'll make the filters return error messages (see bottom)
    HAS_NETADDR = False
else:

    class mac_linux(netaddr.mac_unix):
        pass

    mac_linux.word_fmt = "%.2x"

DOCUMENTATION = """
    name: ipv6
    author: Ashwini Mhatre (@amhatre)
    version_added: "2.5.0"
    short_description: To filter only Ipv6 addresses Ipv6 filter is used.
    description:
        - Sometimes you need only IPv6 addresses. To filter only Ipv6 addresses Ipv6 filter is used.
    options:
        value:
            description:
            - list of subnets or individual address or any other values input for ipv6 plugin
            type: raw
            required: True
        query:
            description:
            - You can provide a single argument to each ipv6() filter.
            - Example. query type 'ipv4' to convert ipv6 into ipv4
            type: str
            default: ''
    notes:
"""

EXAMPLES = r"""
#### examples
# Ipv6 filter plugin with different queries.
- name: Set value as input list
  ansible.builtin.set_fact:
    value:
      - 192.24.2.1
      - ::ffff:192.168.32.0/120
      - ''
      - ::ffff:192.24.2.1/128
      - 192.168.32.0/24
      - fe80::100/10
      - true
- name: IPv6 filter to filter Ipv6 Address
  debug:
    msg: "{{ value|ansible.utils.ipv6 }}"

- name: convert IPv6 addresses into IPv4 addresses.
  debug:
    msg: "{{ value|ansible.utils.ipv6('ipv4') }}"

- name: filter only  IPv6 addresses.
  debug:
    msg: "{{ value|ansible.utils.ipv6('address') }}"


# PLAY [Ipv6 filter plugin with different queries.] ******************************************************************
# TASK [Set value as input list] ***************************************************************************************
# ok: [localhost] => {
#     "ansible_facts": {
#         "value": [
#             "192.24.2.1",
#             "::ffff:192.168.32.0/120",
#             "",
#             "::ffff:192.24.2.1/128",
#             "192.168.32.0/24",
#             "fe80::100/10",
#             true
#         ]
#     },
#     "changed": false
# }
#
# TASK [IPv6 filter to filter Ipv6 Address] ****************************************************************************
# ok: [localhost] => {
#     "msg": [
#         "::ffff:192.168.32.0/120",
#         "::ffff:192.24.2.1/128",
#         "fe80::100/10"
#     ]
# }
#
# TASK [convert IPv6 addresses into IPv4 addresses.] *******************************************************************
# ok: [localhost] => {
#     "msg": [
#         "192.168.32.0/24",
#         "192.24.2.1/32"
#     ]
# }
#
# TASK [filter only  IPv6 addresses] *******************************************************************
# ok: [localhost] => {
#     "msg": [
#         "::ffff:192.168.32.0",
#         "::ffff:192.24.2.1",
#         "fe80::100"
#     ]
# }
#
"""

RETURN = """
  data:
    type: raw
    description:
      - Returns values valid for a particular query.
"""


@pass_environment
def _ipv6(*args, **kwargs):
    """This filter is designed to return the input value if a query is True, and False if a query is False"""
    keys = ["value", "query"]
    data = dict(zip(keys, args[1:]))
    data.update(kwargs)
    try:
        if isinstance(data["value"], str):
            pass
        elif isinstance(data["value"], list):
            pass
        elif isinstance(data["value"], int):
            pass
        else:
            raise AnsibleFilterError(
                "Unrecognized type <{0}> for ipv6 filter <{1}>".format(
                    type(data["value"]),
                    "value",
                ),
            )

    except (TypeError, ValueError):
        raise AnsibleFilterError(
            "Unrecognized type <{0}> for ipv6 filter <{1}>".format(type(data["value"]), "value"),
        )
    aav = AnsibleArgSpecValidator(data=data, schema=DOCUMENTATION, name="ipv6")
    valid, errors, updated_data = aav.validate()
    if not valid:
        raise AnsibleFilterError(errors)
    return ipv6(**updated_data)


def ipv6(value, query=""):
    return ipaddr(value, query, version=6, alias="ipv6")


class FilterModule(object):
    """IP address and network manipulation filters"""

    filter_map = {
        # IP addresses and networks
        "ipv6": _ipv6,
    }

    def filters(self):
        """ipv6 filter"""
        if HAS_NETADDR:
            return self.filter_map
        else:
            return dict((f, partial(_need_netaddr, f)) for f in self.filter_map)



#!/usr/bin/python
# coding: utf-8 -*-

# (c) 2017, Wayne Witzel III <wayne@riotousliving.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ['preview'], 'supported_by': 'community'}


DOCUMENTATION = '''
---
module: group
author: "Wayne Witzel III (@wwitzel3)"
short_description: create, update, or destroy Automation Platform Controller group.
description:
    - Create, update, or destroy Automation Platform Controller groups. See
      U(https://www.ansible.com/tower) for an overview.
options:
    name:
      description:
        - The name to use for the group.
      required: True
      type: str
    description:
      description:
        - The description to use for the group.
      type: str
    inventory:
      description:
        - Inventory name, ID, or named URL that the group should be made a member of.
      required: True
      type: str
    variables:
      description:
        - Variables to use for the group.
      type: dict
    hosts:
      description:
        - List of host names, IDs, or named URLs that should be put in this group.
      type: list
      elements: str
    children:
      description:
        - List of groups names, IDs, or named URLs that should be nested inside in this group.
      type: list
      elements: str
      aliases:
        - groups
    preserve_existing_hosts:
      description:
        - Provide option (False by default) to preserves existing hosts in an existing group.
      default: False
      type: bool
    preserve_existing_children:
      description:
        - Provide option (False by default) to preserves existing children in an existing group.
      default: False
      type: bool
      aliases:
        - preserve_existing_groups
    state:
      description:
        - Desired state of the resource.
      default: "present"
      choices: ["present", "absent", "exists"]
      type: str
    new_name:
      description:
        - A new name for this group (for renaming)
      type: str
extends_documentation_fragment: awx.awx.auth
'''


EXAMPLES = '''
- name: Add group
  group:
    name: localhost
    description: "Local Host Group"
    inventory: "Local Inventory"
    state: present
    controller_config_file: "~/tower_cli.cfg"

- name: Add group
  group:
    name: Cities
    description: "Local Host Group"
    inventory: Default Inventory
    hosts:
      - fda
    children:
      - NewYork
    preserve_existing_hosts: True
    preserve_existing_children: True
'''

from ..module_utils.controller_api import ControllerAPIModule
import json


def main():
    # Any additional arguments that are not fields of the item can be added here
    argument_spec = dict(
        name=dict(required=True),
        new_name=dict(),
        description=dict(),
        inventory=dict(required=True),
        variables=dict(type='dict'),
        hosts=dict(type='list', elements='str'),
        children=dict(type='list', elements='str', aliases=['groups']),
        preserve_existing_hosts=dict(type='bool', default=False),
        preserve_existing_children=dict(type='bool', default=False, aliases=['preserve_existing_groups']),
        state=dict(choices=['present', 'absent', 'exists'], default='present'),
    )

    # Create a module for ourselves
    module = ControllerAPIModule(argument_spec=argument_spec)

    # Extract our parameters
    name = module.params.get('name')
    new_name = module.params.get('new_name')
    inventory = module.params.get('inventory')
    description = module.params.get('description')
    state = module.params.pop('state')
    preserve_existing_hosts = module.params.get('preserve_existing_hosts')
    preserve_existing_children = module.params.get('preserve_existing_children')
    variables = module.params.get('variables')

    # Attempt to look up the related items the user specified (these will fail the module if not found)
    inventory_id = module.resolve_name_to_id('inventories', inventory)

    # Attempt to look up the object based on the provided name and inventory ID
    group = module.get_one('groups', name_or_id=name, check_exists=(state == 'exists'), **{'data': {'inventory': inventory_id}})

    if state == 'absent':
        # If the state was absent we can let the module delete it if needed, the module will handle exiting from this
        module.delete_if_needed(group)

    # Create the data that gets sent for create and update
    group_fields = {
        'name': new_name if new_name else (module.get_item_name(group) if group else name),
        'inventory': inventory_id,
    }
    if description is not None:
        group_fields['description'] = description
    if variables is not None:
        group_fields['variables'] = json.dumps(variables)

    association_fields = {}
    for resource, relationship in (('hosts', 'hosts'), ('groups', 'children')):
        name_list = module.params.get(relationship)
        if name_list is None:
            continue
        id_list = []
        for sub_name in name_list:
            sub_obj = module.get_one(
                resource,
                name_or_id=sub_name,
                **{
                    'data': {'inventory': inventory_id},
                }
            )
            if sub_obj is None:
                module.fail_json(msg='Could not find {0} with name {1}'.format(resource, sub_name))
            id_list.append(sub_obj['id'])
        # Preserve existing objects
        if (preserve_existing_hosts and relationship == 'hosts') or (preserve_existing_children and relationship == 'children'):
            if group:
                preserve_existing_check = module.get_all_endpoint(group['related'][relationship])
                for sub_obj in preserve_existing_check['json']['results']:
                    if 'id' in sub_obj:
                        id_list.append(sub_obj['id'])
        if id_list:
            association_fields[relationship] = id_list

    # If the state was present we can let the module build or update the existing group, this will return on its own
    module.create_or_update_if_needed(group, group_fields, endpoint='groups', item_type='group', associations=association_fields)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-sun (@Fred-sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_postgresqlflexibleserver
version_added: "2.2.0"
short_description: Manage PostgreSQL Flexible Server instance
description:
    - Create, update and delete instance of PostgreSQL Flexible Server.

options:
    resource_group:
        description:
            - The name of the resource group that contains the resource.
            - You can obtain this value from the Azure Resource Manager API or the portal.
        required: True
        type: str
    name:
        description:
            - The name of the flexible server.
        required: True
        type: str
    sku:
        description:
            - The SKU (pricing tier) of the server.
        type: dict
        suboptions:
            name:
                description:
                    - The name of the sku, typically, tier + family + cores, such as Standard_D4s_v3.
                type: str
                required: True
            tier:
                description:
                    - The tier of the particular
                type: str
                choices:
                    - Burstable
                    - GeneralPurpose
                    - MemoryOptimized
                required: True
    location:
        description:
            - Resource location. If not set, location from the resource group will be used as default.
        type: str
    storage:
        description:
            - Storage properties of a server.
        type: dict
        suboptions:
            storage_size_gb:
                description:
                    - The storage size for the server.
                type: int
    administrator_login:
        description:
            - The administrator's login name of a server.
            - Can only be specified when the server is being created (and is required for creation).
        type: str
    administrator_login_password:
        description:
            - The administrator login password (required for server creation).
        type: str
    version:
        description:
            - PostgreSQL Server version.
        type: str
        choices:
            - '11'
            - '12'
            - '13'
            - '14'
            - '15'
            - '16'
    fully_qualified_domain_name:
        description:
            - The fully qualified domain name of a server.
        type: str
    backup:
        description:
            - Backup properties of a server.
        type: dict
        suboptions:
            backup_retention_days:
                description:
                    - Backup retention days for the server.
                type: int
            geo_redundant_backup:
                description:
                    - A value indicating whether Geo-Redundant backup is enabled on the server.
                type: str
                choices:
                    - Enabled
                    - Disabled
    network:
        description:
            - Network properties of a server.
        type: dict
        suboptions:
            delegated_subnet_resource_id:
                description:
                    - Delegated subnet arm resource id.
                type: str
            private_dns_zone_arm_resource_id:
                description:
                    - Private dns zone arm resource id.
                type: str
            public_network_access:
                description:
                    - Public network access is enabled or not.
                type: str
                choices:
                    - Enabled
                    - Disabled
    high_availability:
        description:
            - High availability properties of a server.
        type: dict
        suboptions:
            mode:
                description:
                    - The HA mode for the server.
                type: str
                choices:
                    - Disabled
                    - ZoneRedundant
            standby_availability_zone:
                description:
                    - Availability zone information of the standby.
                type: str
    maintenance_window:
        description:
            - Maintenance window properties of a server.
        type: dict
        suboptions:
            custom_window:
                description:
                    - Indicates whether custom window is enabled or disabled.
                type: str
            start_hour:
                description:
                    - Start hour for maintenance window.
                type: int
            start_minute:
                description:
                    - Start minute for maintenance window.
                type: int
            day_of_week:
                description:
                    - Day of week for maintenance window.
                type: int
    point_in_time_utc:
        description:
            - Restore point creation time (ISO8601 format), specifying the time to restore from.
            - It's required when I(create_mode=PointInTimeRestore).
        type: str
    availability_zone:
        description:
            - Availability zone information of the server
        type: str
    create_mode:
        description:
            - The mode to create a new PostgreSQL server.
        type: str
        choices:
            - Default
            - Create
            - Update
            - PointInTimeRestore
    source_server_resource_id:
        description:
            - The source server resource ID to restore from.
            - It's required when I(create_mode=PointInTimeRestore)
        type: str
    state:
        description:
            - Assert the state of the PostgreSQL Flexible server.
            - Use C(present) to create or update a server and C(absent) to delete it.
        default: present
        type: str
        choices:
            - present
            - absent
    is_restart:
        description:
            - Whether to restart the Post gresql server.
        type: bool
        default: False
    is_stop:
        description:
            - Whether to stop the Post gresql server.
        type: bool
        default: False
    is_start:
        description:
            - Whether to start the Post gresql server.
        type: bool
        default: False
    identity:
        description:
            - Identity for the Server.
        type: dict
        version_added: '2.4.0'
        suboptions:
            type:
                description:
                    - Type of the managed identity
                required: false
                choices:
                    - UserAssigned
                    - None
                default: None
                type: str
            user_assigned_identities:
                description:
                    - User Assigned Managed Identities and its options
                required: false
                type: dict
                default: {}
                suboptions:
                    id:
                        description:
                            - List of the user assigned identities IDs associated to the VM
                        required: false
                        type: list
                        elements: str
                        default: []
                    append:
                        description:
                            - If the list of identities has to be appended to current identities (true) or if it has to replace current identities (false)
                        required: false
                        type: bool
                        default: True


extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)

'''

EXAMPLES = '''
- name: Create (or update) PostgreSQL Flexible Server
  azure_rm_postgresqlflexibleserver:
    resource_group: myResourceGroup
    name: testserver
    sku:
      name: Standard_B1ms
      tier: Burstable
    administrator_login: azureuser
    administrator_login_password: "{{ password }}"
    version: 12
    storage:
      storage_size_gb: 128
    fully_qualified_domain_name: st-private-dns-zone.postgres.database.azure.com
    backup:
      backup_retention_days: 7
      geo_redundant_backup: Disabled
    maintenance_window:
      custom_window: Enabled
      start_hour: 8
      start_minute: 0
      day_of_week: 0
    point_in_time_utc: 2023-05-31T00:28:17.7279547+00:00
    availability_zone: 1
    create_mode: Default

- name: Delete PostgreSQL Flexible Server
  azure_rm_postgresqlflexibleserver:
    resource_group: myResourceGroup
    name: testserver
    state: absent
'''

RETURN = '''
servers:
    description:
        - A list of dictionaries containing facts for PostgreSQL Flexible servers.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID of the postgresql flexible server.
            returned: always
            type: str
            sample: "/subscriptions/xxx/resourceGroups/myResourceGroup/providers/Microsoft.DBforPostgreSQL/flexibleservers/postgresql3"
        resource_group:
            description:
                - Resource group name.
            returned: always
            type: str
            sample: myResourceGroup
        name:
            description:
                - Resource name.
            returned: always
            type: str
            sample: postgreabdud1223
        location:
            description:
                - The location the resource resides in.
            returned: always
            type: str
            sample: eastus
        sku:
            description:
                - The SKU of the server.
            returned: always
            type: complex
            contains:
                name:
                    description:
                        - The name of the SKU.
                    returned: always
                    type: str
                    sample: Standard_B1ms
                tier:
                    description:
                        - The tier of the particular SKU.
                    returned: always
                    type: str
                    sample: Burstable
        storage:
            description:
                - The maximum storage allowed for a server.
            returned: always
            type: complex
            contains:
                storage_size_gb:
                    description:
                        - ax storage allowed for a server.
                    type: int
                    returned: always
                    sample: 128
        administrator_login:
            description:
                - The administrator's login name of a server.
            returned: always
            type: str
            sample: azureuser
        version:
            description:
                - Flexible Server version.
            returned: always
            type: str
            sample: "12"
            choices:
                - '11'
                - '12'
                - '13'
        fully_qualified_domain_name:
            description:
                - The fully qualified domain name of the flexible server.
            returned: always
            type: str
            sample: postflexiblefredpgsqlflexible.postgres.database.azure.com
        availability_zone:
            description:
                - Availability zone information of the server.
            type: str
            returned: always
            sample: 1
        backup:
            description:
                - Backup properties of a server.
            type: complex
            returned: always
            contains:
                backup_retention_days:
                    description:
                        - Backup retention days for the server.
                    type: int
                    returned: always
                    sample: 7
                geo_redundant_backup:
                    description:
                        - A value indicating whether Geo-Redundant backup is enabled on the server.
                    type: str
                    returned: always
                    sample: Disabled
        high_availability:
            description:
                - High availability properties of a server.
            type: complex
            returned: always
            contains:
                mode:
                    description:
                        - The HA mode for the server.
                    returned: always
                    sample: Disabled
                    type: str
                standby_availability_zone:
                    description:
                        - availability zone information of the standby.
                    type: str
                    returned: always
                    sample: null
        maintenance_window:
            description:
                - Maintenance window properties of a server.
            type: complex
            returned: always
            contains:
                custom_window:
                    description:
                        - Indicates whether custom window is enabled or disabled.
                    returned: always
                    sample: Enabled
                    type: str
                day_of_week:
                    description:
                        - Day of week for maintenance window.
                    returned: always
                    sample: 0
                    type: int
                start_hour:
                    description:
                        - Start hour for maintenance window.
                    type: int
                    returned: always
                    sample: 8
                start_minute:
                    description:
                        - Start minute for maintenance window.
                    type: int
                    returned: always
                    sample: 0
        network:
            description:
                - Network properties of a server.
            type: complex
            returned: always
            contains:
                delegated_subnet_resource_id:
                    description:
                        - Delegated subnet arm resource id.
                    type: str
                    returned: always
                    sample: null
                private_dns_zone_arm_resource_id:
                    description:
                        - Private dns zone arm resource id.
                    type: str
                    returned: always
                    sample: null
                public_network_access:
                    description:
                        - Public network access is enabled or not.
                    type: str
                    returned: always
                    sample: Enabled
        point_in_time_utc:
            description:
                - Restore point creation time (ISO8601 format).
            type: str
            sample: null
            returned: always
        source_server_resource_id:
            description:
                - The source server resource ID to restore from.
            type: str
            returned: always
            sample: null
        system_data:
            description:
                - The system metadata relating to this resource.
            type: complex
            returned: always
            contains:
                created_by:
                    description:
                        - The identity that created the resource.
                    type: str
                    returned: always
                    sample: null
                created_by_type:
                    description:
                        - The type of identity that created the resource.
                    returned: always
                    type: str
                    sample: null
                created_at:
                    description:
                        - The timestamp of resource creation (UTC).
                    returned: always
                    sample: null
                    type: str
                last_modified_by:
                    description:
                        - The identity that last modified the resource.
                    type: str
                    returned: always
                    sample: null
                last_modified_by_type:
                    description:
                        - The type of identity that last modified the resource.
                    returned: always
                    sample: null
                    type: str
                last_modified_at:
                    description:
                        - The timestamp of resource last modification (UTC).
                    returned: always
                    sample: null
                    type: str
        tags:
            description:
                - Tags assigned to the resource. Dictionary of string:string pairs.
            type: dict
            returned: always
            sample: { tag1: abc }
'''


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBaseExt
    import azure.mgmt.rdbms.postgresql_flexibleservers.models as PostgreSQLFlexibleModels
    from azure.core.exceptions import ResourceNotFoundError
    from azure.core.polling import LROPoller
except ImportError:
    # This is handled in azure_rm_common
    pass


sku_spec = dict(
    name=dict(type='str', required=True),
    tier=dict(type='str', required=True, choices=["Burstable", "GeneralPurpose", "MemoryOptimized"])
)


maintenance_window_spec = dict(
    custom_window=dict(type='str'),
    start_hour=dict(type='int'),
    start_minute=dict(type='int'),
    day_of_week=dict(type='int'),
)


high_availability_spec = dict(
    mode=dict(type='str', choices=["Disabled", "ZoneRedundant"]),
    standby_availability_zone=dict(type='str')
)


network_spec = dict(
    delegated_subnet_resource_id=dict(type='str'),
    private_dns_zone_arm_resource_id=dict(type='str'),
    public_network_access=dict(type='str', choices=["Enabled", "Disabled"])
)


backup_spec = dict(
    backup_retention_days=dict(type='int'),
    geo_redundant_backup=dict(type='str', choices=["Enabled", "Disabled"]),
)


storage_spec = dict(
    storage_size_gb=dict(type='int')
)


user_assigned_identities_spec = dict(
    id=dict(type='list', default=[], elements='str'),
    append=dict(type='bool', default=True)
)


managed_identity_spec = dict(
    type=dict(type='str', choices=['UserAssigned', 'None'], default='None'),
    user_assigned_identities=dict(type='dict', options=user_assigned_identities_spec, default={}),
)


class AzureRMPostgreSqlFlexibleServers(AzureRMModuleBaseExt):
    """Configuration class for an Azure RM PostgreSQL Flexible Server resource"""

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            sku=dict(
                type='dict',
                options=sku_spec
            ),
            location=dict(
                type='str'
            ),
            administrator_login=dict(
                type='str'
            ),
            administrator_login_password=dict(
                type='str',
                no_log=True
            ),
            version=dict(
                type='str',
                choices=['11', '12', '13', '14', '15', '16']
            ),
            fully_qualified_domain_name=dict(
                type='str',
            ),
            storage=dict(
                type='dict',
                options=storage_spec
            ),
            backup=dict(
                type='dict',
                options=backup_spec
            ),
            network=dict(
                type='dict',
                options=network_spec
            ),
            high_availability=dict(
                type='dict',
                options=high_availability_spec
            ),
            maintenance_window=dict(
                type='dict',
                options=maintenance_window_spec
            ),
            point_in_time_utc=dict(
                type='str'
            ),
            availability_zone=dict(
                type='str'
            ),
            create_mode=dict(
                type='str',
                choices=['Default', 'Create', 'Update', 'PointInTimeRestore']
            ),
            is_start=dict(
                type='bool',
                default=False,
            ),
            is_restart=dict(
                type='bool',
                default=False
            ),
            is_stop=dict(
                type='bool',
                default=False
            ),
            source_server_resource_id=dict(
                type='str'
            ),
            identity=dict(type='dict', options=managed_identity_spec),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.resource_group = None
        self.name = None
        self.parameters = dict()
        self.update_parameters = dict()
        self.tags = None
        self.is_start = None
        self.is_stop = None
        self.is_restart = None
        self.identity = None

        self.results = dict(changed=False)
        self.state = None

        self._managed_identity = None

        super(AzureRMPostgreSqlFlexibleServers, self).__init__(derived_arg_spec=self.module_arg_spec,
                                                               supports_check_mode=True,
                                                               supports_tags=True)

    @property
    def managed_identity(self):
        if not self._managed_identity:
            self._managed_identity = {"identity": PostgreSQLFlexibleModels.UserAssignedIdentity,
                                      "user_assigned": PostgreSQLFlexibleModels.UserIdentity
                                      }
        return self._managed_identity

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            if hasattr(self, key):
                setattr(self, key, kwargs[key])
            elif kwargs[key] is not None:
                self.parameters[key] = kwargs[key]
                for key in ['location', 'sku', 'administrator_login_password', 'storage', 'backup', 'high_availability', 'maintenance_window', 'create_mode']:
                    self.update_parameters[key] = kwargs[key]

        old_response = None
        response = None
        changed = False

        resource_group = self.get_resource_group(self.resource_group)

        if "location" not in self.parameters:
            self.parameters["location"] = resource_group.location
            self.update_parameters["location"] = resource_group.location

        old_response = self.get_postgresqlflexibleserver()

        if not old_response:
            self.log("PostgreSQL Flexible Server instance doesn't exist")
            if self.state == 'present':
                if not self.check_mode:
                    if self.identity:
                        update_identity, new_identity = self.update_managed_identity(new_identity=self.identity)
                        if update_identity:
                            self.parameters['identity'] = new_identity
                    response = self.create_postgresqlflexibleserver(self.parameters)
                    if self.is_stop:
                        self.stop_postgresqlflexibleserver()
                    elif self.is_start:
                        self.start_postgresqlflexibleserver()
                    elif self.is_restart:
                        self.restart_postgresqlflexibleserver()
                changed = True
            else:
                self.log("PostgreSQL Flexible Server instance doesn't exist, Don't need to delete")
        else:
            self.log("PostgreSQL Flexible Server instance already exists")
            if self.state == 'present':
                update_flag = False
                if self.update_parameters.get('sku') is not None:
                    for key in self.update_parameters['sku'].keys():
                        if self.update_parameters['sku'][key] is not None and self.update_parameters['sku'][key] != old_response['sku'].get(key):
                            update_flag = True
                        else:
                            self.update_parameters['sku'][key] = old_response['sku'].get(key)

                if self.update_parameters.get('storage') is not None and self.update_parameters['storage'] != old_response['storage']:
                    update_flag = True
                else:
                    self.update_parameters['storage'] = old_response['storage']

                if self.update_parameters.get('backup') is not None:
                    for key in self.update_parameters['backup'].keys():
                        if self.update_parameters['backup'][key] is not None and self.update_parameters['backup'][key] != old_response['backup'].get(key):
                            update_flag = True
                        else:
                            self.update_parameters['backup'][key] = old_response['backup'].get(key)

                if self.update_parameters.get('high_availability') is not None:
                    for key in self.update_parameters['high_availability'].keys():
                        if (self.update_parameters['high_availability'][key] is not None) and\
                                (self.update_parameters['high_availability'][key] != old_response['high_availability'].get(key)):
                            update_flag = True
                        else:
                            self.update_parameters['high_availability'][key] = old_response['high_availability'].get(key)

                if self.update_parameters.get('maintenance_window') is not None:
                    for key in self.update_parameters['maintenance_window'].keys():
                        if (self.update_parameters['maintenance_window'][key] is not None) and\
                                (self.update_parameters['maintenance_window'][key] != old_response['maintenance_window'].get(key)):
                            update_flag = True
                        else:
                            self.update_parameters['maintenance_window'][key] = old_response['maintenance_window'].get(key)

                if self.identity:
                    update_identity, new_identity = self.update_managed_identity(new_identity=self.identity,
                                                                                 curr_identity=old_response.get('identity', {}))
                    if update_identity:
                        self.update_parameters['identity'] = new_identity
                        update_flag = True

                update_tags, new_tags = self.update_tags(old_response['tags'])
                self.update_parameters['tags'] = new_tags
                if update_tags:
                    update_flag = True

                if update_flag:
                    changed = True
                    if not self.check_mode:
                        response = self.update_postgresqlflexibleserver(self.update_parameters)
                    else:
                        response = old_response
                    if self.is_stop:
                        self.stop_postgresqlflexibleserver()
                        changed = True
                    elif self.is_start:
                        self.start_postgresqlflexibleserver()
                        changed = True
                    elif self.is_restart:
                        self.restart_postgresqlflexibleserver()
                        changed = True
                else:
                    if not self.check_mode:
                        if self.is_stop:
                            self.stop_postgresqlflexibleserver()
                            changed = True
                        elif self.is_start:
                            self.start_postgresqlflexibleserver()
                            changed = True
                        elif self.is_restart:
                            self.restart_postgresqlflexibleserver()
                            changed = True
                    response = old_response
            else:
                self.log("PostgreSQL Flexible Server instance already exist, will be deleted")
                changed = True
                if not self.check_mode:
                    response = self.delete_postgresqlflexibleserver()

        self.results['changed'] = changed
        self.results['state'] = response

        return self.results

    def update_postgresqlflexibleserver(self, body):
        '''
        Updates PostgreSQL Flexible Server with the specified configuration.
        :return: deserialized PostgreSQL Flexible Server instance state dictionary
        '''
        self.log("Updating the PostgreSQL Flexible Server instance {0}".format(self.name))
        try:
            # structure of parameters for update must be changed
            response = self.postgresql_flexible_client.servers.begin_update(resource_group_name=self.resource_group,
                                                                            server_name=self.name,
                                                                            parameters=body)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the PostgreSQL Flexible Server instance.')
            self.fail("Error updating the PostgreSQL Flexible Server instance: {0}".format(str(exc)))
        return self.format_item(response)

    def create_postgresqlflexibleserver(self, body):
        '''
        Creates PostgreSQL Flexible Server with the specified configuration.
        :return: deserialized PostgreSQL Flexible Server instance state dictionary
        '''
        self.log("Creating the PostgreSQL Flexible Server instance {0}".format(self.name))
        try:
            response = self.postgresql_flexible_client.servers.begin_create(resource_group_name=self.resource_group,
                                                                            server_name=self.name,
                                                                            parameters=body)
            if isinstance(response, LROPoller):
                response = self.get_poller_result(response)

        except Exception as exc:
            self.log('Error attempting to create the PostgreSQL Flexible Server instance.')
            self.fail("Error creating the PostgreSQL Flexible Server instance: {0}".format(str(exc)))
        return self.format_item(response)

    def delete_postgresqlflexibleserver(self):
        '''
        Deletes specified PostgreSQL Flexible Server instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the PostgreSQL Flexible Server instance {0}".format(self.name))
        try:
            self.postgresql_flexible_client.servers.begin_delete(resource_group_name=self.resource_group,
                                                                 server_name=self.name)
        except Exception as e:
            self.log('Error attempting to delete the PostgreSQL Flexible Server instance.')
            self.fail("Error deleting the PostgreSQL Flexible Server instance: {0}".format(str(e)))

    def stop_postgresqlflexibleserver(self):
        '''
        Stop PostgreSQL Flexible Server instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Stop the PostgreSQL Flexible Server instance {0}".format(self.name))
        try:
            self.postgresql_flexible_client.servers.begin_stop(resource_group_name=self.resource_group,
                                                               server_name=self.name)
        except Exception as e:
            self.log('Error attempting to stop the PostgreSQL Flexible Server instance.')
            self.fail("Error stop the PostgreSQL Flexible Server instance: {0}".format(str(e)))

    def start_postgresqlflexibleserver(self):
        '''
        Start PostgreSQL Flexible Server instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Starting the PostgreSQL Flexible Server instance {0}".format(self.name))
        try:
            self.postgresql_flexible_client.servers.begin_start(resource_group_name=self.resource_group,
                                                                server_name=self.name)
        except Exception as e:
            self.log('Error attempting to start the PostgreSQL Flexible Server instance.')
            self.fail("Error starting the PostgreSQL Flexible Server instance: {0}".format(str(e)))

    def restart_postgresqlflexibleserver(self):
        '''
        Restart PostgreSQL Flexible Server instance in the specified subscription and resource group.

        :return: True
        '''
        self.log("Restarting the PostgreSQL Flexible Server instance {0}".format(self.name))
        try:
            self.postgresql_flexible_client.servers.begin_restart(resource_group_name=self.resource_group,
                                                                  server_name=self.name)
        except Exception as e:
            self.log('Error attempting to restart the PostgreSQL Flexible Server instance.')
            self.fail("Error restarting the PostgreSQL Flexible Server instance: {0}".format(str(e)))

    def get_postgresqlflexibleserver(self):
        '''
        Gets the properties of the specified PostgreSQL Flexible Server.

        :return: deserialized PostgreSQL Flexible Server instance state dictionary
        '''
        self.log("Checking if the PostgreSQL Flexible Server instance {0} is present".format(self.name))
        try:
            response = self.postgresql_flexible_client.servers.get(resource_group_name=self.resource_group,
                                                                   server_name=self.name)
            self.log("Response : {0}".format(response))
            self.log("PostgreSQL Flexible Server instance : {0} found".format(response.name))
        except ResourceNotFoundError as e:
            self.log('Did not find the PostgreSQL Flexible Server instance. Exception as {0}'.format(str(e)))
            return None

        return self.format_item(response)

    def format_item(self, item):
        result = dict(
            id=item.id,
            resource_group=self.resource_group,
            name=item.name,
            sku=dict(),
            location=item.location,
            tags=item.tags,
            system_data=dict(),
            administrator_login=item.administrator_login,
            version=item.version,
            minor_version=item.minor_version,
            fully_qualified_domain_name=item.fully_qualified_domain_name,
            storage=dict(),
            backup=dict(),
            network=dict(),
            high_availability=dict(),
            maintenance_window=dict(),
            source_server_resource_id=item.source_server_resource_id,
            point_in_time_utc=item.point_in_time_utc,
            availability_zone=item.availability_zone,
        )
        if item.sku is not None:
            result['sku']['name'] = item.sku.name
            result['sku']['tier'] = item.sku.tier
        if item.system_data is not None:
            result['system_data']['created_by'] = item.system_data.created_by
            result['system_data']['created_by_type'] = item.system_data.created_by_type
            result['system_data']['created_at'] = item.system_data.created_at
            result['system_data']['last_modified_by'] = item.system_data.last_modified_by
            result['system_data']['last_modified_by_type'] = item.system_data.last_modified_by_type
            result['system_data']['last_modified_at'] = item.system_data.last_modified_at
        if item.storage is not None:
            result['storage']['storage_size_gb'] = item.storage.storage_size_gb
        if item.backup is not None:
            result['backup']['backup_retention_days'] = item.backup.backup_retention_days
            result['backup']['geo_redundant_backup'] = item.backup.geo_redundant_backup
        if item.network is not None:
            result['network']['public_network_access'] = item.network.public_network_access
            result['network']['delegated_subnet_resource_id'] = item.network.delegated_subnet_resource_id
            result['network']['private_dns_zone_arm_resource_id'] = item.network.private_dns_zone_arm_resource_id
        if item.high_availability is not None:
            result['high_availability']['mode'] = item.high_availability.mode
            result['high_availability']['standby_availability_zone'] = item.high_availability.standby_availability_zone
        if item.maintenance_window is not None:
            result['maintenance_window']['custom_window'] = item.maintenance_window.custom_window
            result['maintenance_window']['start_minute'] = item.maintenance_window.start_minute
            result['maintenance_window']['start_hour'] = item.maintenance_window.start_hour
            result['maintenance_window']['day_of_week'] = item.maintenance_window.day_of_week
        if item.identity is not None:
            result['identity'] = item.identity.as_dict()
        else:
            result['identity'] = PostgreSQLFlexibleModels.UserAssignedIdentity(type='None').as_dict()

        return result


def main():
    """Main execution"""
    AzureRMPostgreSqlFlexibleServers()


if __name__ == '__main__':
    main()
